PRO PLOT_DROP,X_ARRAY,Y_ARRAY,DROP_VALUE,TYPE,$
    symsize=smsz,psym=psm,linestyle=lnstl,noclip=noclip,color=color,$
    thickness=thck
;+
; NAME:
;	   PLOT_DROP
; PURPOSE:
;	Plots arrays where some points have data dropout.  Only those segments
;	with data are plotted.  If !PSYM is not positive, then lines will
;	connect the data points only within complete segments and will not
;	cross over segments with data dropout.
;  DROP criterion modifed by HUGHES, so all data LT DROP accepted, where
;  DROP defaults to 99999.0
; CATEGORY:
; CALLING SEQUENCE:
;	   PLOT_DROP,  [ X_ARRAY, ]  Y_ARRAY  [, DROP_VALUE  [, TYPE ]]
; INPUTS:
;	   Y_ARRAY		- Array of y values, including data dropouts.
; OPTIONAL INPUT PARAMETERS:
;	   X_ARRAY		- Array of x values, defaults to indices of Y_ARRAY.
;	   DROP_VALUE	- Values of Y_ARRAY GE to DROP_VALUE signal the
;			           data dropout.  Defaults to 99999.0.
;	   TYPE		- Use the following values for TYPE to produce
;		      	  logarithmic plots, or control the plot through the
;			        system variable !TYPE.
;
;		TYPE = 0	linear scaling in X and Y
;		TYPE = 1	logarithmic scaling in X
;		TYPE = 2	logarithmic scaling in Y
;		TYPE = 3	logarithmic scaling in X and Y
; KEYWORD PARAMETERS: 
;   PSYM   - determines symbol type to be used for these points
;   SYMSIZE - controls the size of the symbols plotted, default = 1.0
;   LINESTYLE - determines the line style to be used to plot data
;   COLOR - determines the line colour to be used to plot data
;   NOCLIP - determines if data point will be clipped (excluded and not
;             plotted) if they are outside the plot box. Default is clipping.
; OUTPUTS:
;	   None.
; OPTIONAL OUTPUT PARAMETERS:
;	   None.
; COMMON BLOCKS:
;	   None.
; SIDE EFFECTS:
;	   None.
; RESTRICTIONS:
;	   None.
; PROCEDURE:
;	   A plot or overplot is done for each segment.
;
;	The normal system variables, such as !XMIN, etc., can be used to
;	control the plot, or one can allow the plot to autoscale in the usual
;	manner.  In the latter case all of X_ARRAY is used to scale the plot
;	in the X direction, even the parts which have data dropout.
;
; MODIFICATION HISTORY:
;	William Thompson	Applied Research Corporation
;	September, 1987		8201 Corporate Drive
;				Landover, MD  20785
;  T.J.HUGHES - HIA/NRCC - 90.12.11 Modified to run under IDL2
;  J.D.Trickey - 13-AUG-1992 added keywords SYMSIZE,PSYM,LINESTYLE,
;                                           NOCLIP,COLOR
;  J.D.Trickey - 20-OCT-1992 made use of TEMPORARY function
;                            added keyword THICKNESS
;                            replaced second half of code with call to
;                            OPLOT_DROP which does the same thing (faster)
;  J.D.Trickey - 16-DEC-1992 a little recoding to speed things up
;-

on_error,2

;      +-------------------------------------------------------------------+
;      |Parse the input parameters. Use TYPE_PASSED to keep track of       |
;      |whether TYPE  was passed or not.                                   |
;      +-------------------------------------------------------------------+
if keyword_set(noclip) then noclip_val=1 else noclip_val=0
If n_elements(drp_fill) ne 0 then fill=drp_fill else fill=99999.0
If keyword_set(smsz) then size=smsz else size=1.0
If n_elements(psm) ne 0 then begin
    psym_old=!p.psym
    !p.psym = psm
endif
If n_elements(lnstl) ne 0 then begin
    linestyle_old=!p.linestyle
    !p.linestyle = lnstl
endif
If n_elements(color) ne 0 then begin
    color_old=!p.color
    !p.color = color
endif
If n_elements(thckr) ne 0 then begin
    thick_old=!p.thick
    !p.thick = thck
endif
;
case n_params(0) of
  1: begin
   	y = x_array
   	x = findgen(n_elements(y))
   	drop = 99999.
   	type = 0
   	type_passed = 0
     end
  2: begin
   	n = n_elements(y_array)
   	if n eq 1 then begin			;Y_ARRAY and DROP_VALUE passed.
   		y = x_array
   		x = findgen(n_elements(y))
   		drop = y_array
   	end else begin				;X_ARRAY and Y_ARRAY passed.
   		x = x_array
   		y = y_array
;      +----------------------------------------------------------+
;      | Modified by HUGHES. Drop default set to 99999.           |
;      +----------------------------------------------------------+
   		drop = 99999.
   	endelse
	   type = 0
   	type_passed = 0
     end
  3: begin
   	n = n_elements(y_array)
   	if n eq 1 then begin			;Y_ARRAY, DROP_VALUE and TYPE.
   		y = x_array
   		x = findgen(n_elements(y))
   		drop = y_array
   		type = drop_value
   		type_passed = 1
   	end else begin				;X_ & Y_ARRAY and DROP_VALUE.
   		x = x_array
   		y = y_array
   		drop = drop_value
   		type = 0
   		type_passed = 0
   	endelse
     end
  4: begin
   	x = x_array
   	y = y_array
   	drop = drop_value
   	type_passed = 1
     end 
  else: begin
        	print,' This procedure must be called with 1-4 parameters:'
       	print,'      [ X_ARRAY ,]   Y_ARRAY   [, DROP_VALUE  [, TYPE ]]'
       	print,' '
       	print,'           TYPE = 0   (X & Y linear)'
       	print,'           TYPE = 1   (X logarithmic)'
       	print,'           TYPE = 2   (Y logarithmic)'
       	print,'           TYPE = 3   (X & Y logarithmic)'
       	return
        end
endcase
;      +-------------------------------------------------------------------+
;      |Check the variable TYPE.                                           |
;      +-------------------------------------------------------------------+
if ((type lt 0) or (type gt 3)) then begin
	print,' TYPE must be in the range 0-3.'
	return
endif

;      +-------------------------------------------------------------------+
;      |Save the variable !TYPE to be restored at the end of the           |
;      |procedure.                                                         |
;      +-------------------------------------------------------------------+
bang_xtype=!x.type
bang_ytype=!y.type
!x.type=long(type) mod 2
!y.type=long(type)/2

;      +-------------------------------------------------------------------+
;      |Save the variables !C and !XMIN, !XMAX, !YMIN, !YMAX to be         |
;      |restored at the end of the procedure.  If not already defined,     |
;      |set the scaling in X.                                              |
;      +-------------------------------------------------------------------+
bang_c = !c
xrange = !x.range  &  x1 = xrange(0)  &  x2 = xrange(1)
yrange = !y.range  &  y1 = yrange(0)  &  y2 = yrange(1)
if x1 eq x2 then begin
   pxxx=where( x lt drop )
	x1 = min([x(pxxx)],max=x2)
endif

;      +-------------------------------------------------------------------+
;      |Find all the points not equal to DROP_VALUE.                       |
;      +-------------------------------------------------------------------+
;w = where(y ne drop)

;      +----------------------------------------------------------+
;      | Modified by HUGHES. Accept all points lt DROP            |
;      +----------------------------------------------------------+
w = where(y lt drop)
if w(0) eq -1 then begin
   plot,[!x.crange(0),!x.crange(0)],[1.,-1],symsize=size,noclip=noclip_val 
	goto,done
endif

;      +-------------------------------------------------------------------+
;      |If not already defined, set the scaling in Y.  Ignore any data     |
;      |dropouts.                                                          |
;      +-------------------------------------------------------------------+
if y1 eq y2 then begin
	y1 = min(y(w),max=y2)
endif
set_xy,x1,x2,y1,y2

;      +-------------------------------------------------------------------+
;      |Define the variables XX and YY to be the first segment without     |
;      |dropouts. I is the next point with a data dropout.                 |
;      +-------------------------------------------------------------------+
i = min(w)
xx = temporary(x(i:*))
yy = temporary(y(i:*))
i = min(temporary(where(yy ge drop))) - 1
!c = 0

case 1 of
  (i eq -2): begin
              if n_elements(xx) gt 1 then begin
                 plot,xx,yy,symsize=size,noclip=noclip_val
              endif else begin
                 plot,[xx,xx],[yy,yy],symsize=size,noclip=noclip_val 
              endelse 
              goto,done
             end
  (i gt 0): plot,xx(0:i),yy(0:i),symsize=size,noclip=noclip_val 
  else: plot,[xx(0),xx(0)],[yy(0),yy(0)],symsize=size,noclip=noclip_val 
endcase

;      +-------------------------------------------------------------------+
;      |The variables XX and YY are the remaining data points.  I is the   |
;      |position of the last point with a dropout.  Do the next segment.   |
;      +-------------------------------------------------------------------+
	i = i + 1
	if i eq n_elements(xx) then goto,done
	xx = temporary(xx(i:*))
	yy = temporary(yy(i:*))
   oplot_drop,xx,yy,drop=drop,symsize=size,noclip=noclip_val 
;      +-------------------------------------------------------------------+
;      |Exit point restore system variables.                               |
;      +-------------------------------------------------------------------+

DONE:
	!c = bang_c
;	!type = bang_type
   !x.type=bang_xtype
   !y.type=bang_ytype
	!x.range = xrange
   !y.range = yrange
;
If n_elements(psm) ne 0 then begin
    !p.psym = psym_old
endif
If n_elements(lnstl) ne 0 then begin
    !p.linestyle = linestyle_old
endif
If n_elements(color) ne 0 then begin
    !p.color = color_old
endif
If n_elements(thck) ne 0 then begin
    !p.thick = thick_old
endif
	return
	end

