FUNCTION POLYFITW,X,Y,W,NDEGREE,YFIT,YBAND,SIGMA,A
;+
 ; NAME:
;	POLYFITW
;PURPOSE:
;	Least square polynomial fit with optional error estimates.
; CATEGORY:
;	?? - CURVE FITTING.
; CALLING SEQUENCE:
;	COEFF = POLYFITW(X,Y,W,NDEGREE [,YFIT,YBAND,SIGMA,A])
 ;INPUTS:
;	X = independent variable vector.
;	Y = dependent variable vector, should be same length as x.
;	W = vector of weights, same length as x and y
;	NDEGREE = degree of polynomial to fit
; OUTPUTS:
;	Function result= Coefficient vector, length NDEGREE+1.
;   OPTIONAL PARAMETERS:
;	YFIT = Vector of calculated Y's.  Has error + or - YBAND.
;	YBAND = Error estimate for each point = 1 sigma
;	SIGMA = Standard deviation in Y units.
;	A = Correlation matrix of the coefficients.
; COMMON BLOCKS:
;	none.
;	SIDE EFFECTS:
;	none.
;MODIFICATION HISTORY:
;	Written by: George Lawrence, LASP, University of Colorado,
;		December, 1981.
;	Adapted to VAX IDL by: David Stern, Jan, 1982.
;	Weights added, April, 1987,  G. Lawrence
;
;-
ON_ERROR,2                      ;RETURN TO CALLER IF AN ERROR OCCURS
N = N_ELEMENTS(X) < N_ELEMENTS(Y) ; SIZE = SMALLER OF X,Y
M = NDEGREE + 1	; # OF ELEMENTS IN COEFF VEC
;
A = DBLARR(M,M) ; LEAST SQUARE MATRIX, WEIGHTED MATRIX
B = DBLARR(M)	; WILL CONTAIN SUM W*Y*X^J
Z = DBLARR(N) + 1.	; BASIS VECTOR FOR CONSTANT TERM
;
A(0,0) = TOTAL(W)
B(0) = TOTAL(W*Y)
;
FOR P = 1,2*NDEGREE DO BEGIN	; POWER LOOP
Z=Z*X	; Z IS NOW X^P
IF P LT M THEN B(P) = TOTAL(W*Y*Z)	; B IS SUM W*Y*X^J
SUM = TOTAL(W*Z)
FOR J = 0 > (P-NDEGREE), NDEGREE < P DO A(J,P-J) = SUM
	END ; END OF P LOOP, CONSTRUCTION OF A AND B
;
A = INVERT(A)
;
C = FLOAT( MAT_MULT(B,A))	; GET COEFF
;
IF ( N_PARAMS(0) LE 3) THEN RETURN,C	; EXIT IF NO ERROR ESTIMATES
;
; COMPUTE OPTIONAL OUTPUT PARAMETERS.
;
YFIT = FLTARR(N)+C(0)	; ONE-SIGMA ERROR ESTIMATES, INIT
FOR K = 1, NDEGREE DO YFIT = YFIT + C(K)*(X^K)	; SUM BASIS VECTORS
;
VAR = TOTAL((YFIT-Y)^2 )/(N-M)	; VARIANCE ESTIMATE, UNBIASED
;
;
SIGMA=SQRT(VAR)
YBAND = FLTARR(N) + A(0,0)
Z=FLTARR(N)+1.
FOR P=1,2*NDEGREE DO BEGIN	; COMPUTE CORRELATED ERROR ESTIMATES ON Y
	Z = Z*X		; Z IS NOW X^P
	SUM = 0.
	FOR J=0 > (P - NDEGREE), NDEGREE  < P DO SUM = SUM + A(J,P-J)
	YBAND = YBAND + SUM * Z	; ADD IN ALL THE ERROR SOURCES
ENDFOR	; END OF P LOOP
	YBAND = YBAND*VAR
	YBAND = SQRT( YBAND )
RETURN,C
END



