;+
; NAME:
;	CADI_SUB
; PURPOSE:
;	Select a subset of a vector of CADI.DATA structures.
; CALLING SEQUENCE:
;	Result = CADI_SUB( DATA, [TRANGE=TRANGE] [,FRANGE=FRANGE] $
;	                        [,HRANGE=HRANGE] [,DRANGE=DRANGE] )
; INPUTS:
;	DATA = Vector of structures with the form of the DATA field of a CADI 
;	   structure. i.e. An element of DATA has an X field, and possibly
;	   another field such as IQ, XAP or KVEC.
;	   Note that the argument is not a CADI structure with a HEADER field,
;	   etc., but a vector of structures such as CADI.DATA.
; KEYWORD PARAMETERS:
;	TRANGE, FRANGE, HRANGE, DRANGE:
;	   Each is a 2-element integer vector which is the first and last of a 
;	   range of indices into the TIMES, FREQS, HEIGHTS, DOPS fields 
;	   respectively of the HEADER structure associated with DATA. All are 
;	   optional.
; OUTPUTS:
;	Result = A vector of structures which is a subset of the DATA vector.
;	   Result contains only those elements DATA(i) for which
;	   TRANGE(0) LE DATA(i).X.T LE TRANGE(1) (if TRANGE is supplied), and 
;	   so on for the F, H and D fields of DATA(i).X.
; OPTIONAL OUTPUT PARAMETERS:
;	WCOUNT = scalar integer, returns the number of elements in Result.
; RESTRICTIONS:
;	The structure of an element of the argument DATA must have the field X,
;	which in turn must have those of the subfields T, F, H and D which are
;	being used for the selection.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 29 Jul 1994: Written.
;	GRANT, 3 Feb 1995: Ensured WCOUNT returned as zero if enter with D_IN
;	   empty.
;-

function cadi_sub, d_in, wcount, $
   trange=trange, frange=frange, hrange=hrange, drange=drange

;%%   trange=trange, tlist=tlist, $
;%%   frange=frange, flist=flist, $
;%%   hrange=hrange, hlist=hlist, $
;%%   drange=drange, dlist=dlist

;%% Haven't implemented TLIST, etc.

;%% Should have set only TRANGE or TLIST, not both, etc.

;%% Should check whichever of T, F, H and D are chosen is in TAG_NAMES(D.X).

;%% Should check TRANGE, etc. has exactly 2 elements.

on_error, 2

d = d_in
null_d = -1L
wcount = 0L

if (size(d))(0) eq 0 then return, null_d	; No elements in D

if n_elements(trange) eq 2 then begin
   w = where( (d.x.t ge trange(0)) and (d.x.t le trange(1)) ,wcount)
   if wcount gt 0 then $
      d = d(w) $
   else $
      return, null_d
   endif

if n_elements(frange) eq 2 then begin
   w = where( (d.x.f ge frange(0)) and (d.x.f le frange(1)) ,wcount)
   if wcount gt 0 then $
      d = d(w) $
   else $
      return, null_d
   endif

if n_elements(hrange) eq 2 then begin
   w = where( (d.x.h ge hrange(0)) and (d.x.h le hrange(1)) ,wcount)
   if wcount gt 0 then $
      d = d(w) $
   else $
      return, null_d
   endif

if n_elements(drange) eq 2 then begin
   w = where( (d.x.d ge drange(0)) and (d.x.d le drange(1)) ,wcount)
   if wcount gt 0 then $
      d = d(w) $
   else $
      return, null_d
   endif

return, d
end
;+
; NAME:
;	CADI_SUB_STRUCT
; PURPOSE:
;	Select a subset of data from a CADI structure.
; CALLING SEQUENCE:
;	Result = CADI_SUB_STRUCT( CADI, [TRANGE=TRANGE] [,FRANGE=FRANGE] $
;	                        [,HRANGE=HRANGE] [,DRANGE=DRANGE] )
; INPUTS:
;	DATA = A variable of CADI structure.
; KEYWORD PARAMETERS:
;	TRANGE, FRANGE, HRANGE, DRANGE:
;	   Each is a 2-element integer vector which is the first and last of a 
;	   range of indices into the TIMES, FREQS, HEIGHTS, DOPS fields 
;	   respectively of CADI.HEADER. All are optional. They are passed to
;	   CADI_SUB by the _EXTRA keyword mechanism.
; OUTPUTS:
;	Result = A variable of CADI structure. Result.DATA is a subset of the
;	   CADI.DATA vector.
;	   Result.DATA contains only those elements CADI.DATA(i) for which
;	   TRANGE(0) LE CADI.DATA(i).X.T LE TRANGE(1) (if TRANGE is supplied),
;	   and so on for the F, H and D fields of CADI.DATA(i).X.
; RESTRICTIONS:
;	The structure of an element of CADI.DATA must have the field X,
;	which in turn must have those of the subfields T, F, H and D which are
;	being used for the selection.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, Jul 1994: Written.
;-

function cadi_sub_struct, cadi, _extra=extra

on_error, 2

data = cadi.data
if keyword_set(extra) then $
   data = cadi_sub(data, ndata, _extra=extra)

return, create_struct( $
   'HEADER_EXISTS', cadi.header_exists, $
   'NFREBINS', cadi.nfrebins, $
   'NDATA', ndata, $
   'DATATYPE', cadi.datatype, $
   'HEADER', cadi.header, $
   'FREBINS', cadi.frebins, $
   'DATA', data)

end
