;+
; NAME:
;	CADI_REDUCETIME
; PURPOSE:
;	Reduce list of times in a CADI structure to those actually used.
;	For FHT data this allows much more efficient plotting of h't images,
;	since one FHT frequency uses typically every third time.
; CALLING SEQUENCE:
;	Result = CADI_REDUCETIME( CADI )
; INPUTS:
;	CADI = A variable of CADI structure.
; OUTPUTS:
;	Result = A variable of CADI structure.
;	   An element of CADI.HEADER.TIMES appears in Result.HEADER.TIMES if and
;	   only if it is referenced by at least one element of CADI.DATA.X.T.
;	   Result.DATA.X.T is set so the same times are still associated with
;	   the elements of the .DATA vector.
; RESTRICTIONS:
;	%% If CADI.NDATA eq 0 then CADI is returned unchanged. Perhaps a more 
;	%% logical way would be to return NTIMES=0L and TIMES=-1L.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 2 Nov 1994: Written.
;-

function cadi_reducetime, cadi

on_error, 2

if (cadi.header_exists eq !false) or (cadi.ndata eq 0) then return, cadi

; Make list of which elements of CADI.HEADER.TIMES are actually used.
;
; Actually, TB.STARTX indexes elements of CADI.DATA.X.T which index
; CADI.HEADER.TIMES.

tb = cadi_tblocks(cadi, ntimes=ntimes_used)

; Create a header with the NTIMES and TIMES fields changed.

header_out = { $
         nantennas:	cadi.header.nantennas, $
         ndops:    	cadi.header.ndops, $
         nheights:	cadi.header.nheights, $
         ntimes:	ntimes_used, $			; New
         nfreqs:	cadi.header.nfreqs, $
         site:		cadi.header.site, $
         time:		cadi.header.time, $
         filetype:	cadi.header.filetype, $
         minheight:	cadi.header.minheight, $
         maxheight:	cadi.header.maxheight, $
         pps:		cadi.header.pps, $
         npulses_avgd:	cadi.header.npulses_avgd, $
         base_thr100:	cadi.header.base_thr100, $
         noise_thr100:	cadi.header.noise_thr100, $
         min_dop_forsave:cadi.header.min_dop_forsave, $
         dtime:		cadi.header.dtime, $
         gain_control:	cadi.header.gain_control, $
         sig_process:	cadi.header.sig_process, $
         dops:		cadi.header.dops, $
         heights:	cadi.header.heights, $
         freqs:		cadi.header.freqs, $
         times:		cadi.header.times( cadi.data( tb.startx ).x.t ), $ ; New
         dir:           cadi.header.dir, $
         file_ext:      cadi.header.file_ext}

; Create the almost finished result structure.

result = create_struct( $
   'HEADER_EXISTS', cadi.header_exists, $
   'NFREBINS', cadi.nfrebins, $
   'NDATA', cadi.ndata, $
   'DATATYPE', cadi.datatype, $
   'HEADER', header_out, $
   'FREBINS', cadi.frebins, $
   'DATA', cadi.data)

; Adjust Result.DATA.X.T to point to the new list of times (HEADER_OUT.TIMES).

for timex=0,ntimes_used-1 do $
   result.data( tb(timex).startx : tb(timex).endx ).x.t = timex

return, result
end
