;+
; NAME:
;       CADI_XAP2KVEC
; PURPOSE:
;	Return k-vectors of CADI sources given cross-amplitudes and phases.
; CALLING SEQUENCE:
;       CADI_KVEC = CADI_XAP2KVEC( CADI_XAP )
; INPUTS:
;       CADI_XAP: Variable of CADI structure containing XAP (cross-amplitude
;	   and cross-phase) data.
; OUTPUTS:
;       CADI_KVEC = Variable of CADI structure containing KVEC (returning
;	   k-vectors of sources) data. CADI_XAP.NDATA equals CADI_KVEC.NDATA.
;	   Each element of CADI_KVEC.DATA is calculated from the corresponding
;          element of CADI_XAP.DATA. The returned k-vectors are unnormalized.
;	   The 3 components are (Kx,Ky,Kz), where X is east, y is north and z
;	   is up.
; PROCEDURE:
;	The source power in CADI_KVEC is set to the geometric mean of the two
;	cross-powers in CADI_XAP. Noise data is not used in calculating the
;	source power.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 9 Sep 1994: Written by adapting CADI_MD_SOURCES.
;-

function cadi_xap2kvec, cadi

on_error, 2

c = 2.998e8		; m/s
minpower = 1e-10	; Effectively infinitely low

; Check parameters and do initial processing
; ------------------------------------------

if cadi.datatype ne 'XAP' then $
   message, 'Argument does not have XAP (cross amplitude and phase) data'
nsources = cadi.ndata
if nsources eq 0 then $
   goto, form_nullresult
header = cadi.header
d = cadi.data
site = cadi_site(header.site, header.time)

; Create output vector of sources
; -------------------------------

sources = replicate( {x:d(0).x, db:0.0, k:fltarr(3)}, nsources)

sources.x = d.x

; Find source power
; -----------------

; Set the source power to be the geometric mean of the two cross-powers,
; normalized to the noise power, and converted to dB. i.e. noise is 0 dB.
;%% sources.db = db( sqrt(xpow(*,0)*xpow(*,1)) / $
;%%    ( float(ft.noise_power) / float(ft.ndops) ), min=min_power)
; Set the source power to be the geometric mean of the two cross-powers,
; and converted to dB. NOT normalized to the noise power.
sources.db = db( sqrt(d.xpower(0)*d.xpower(1)), min=min_power)

; Find source k-vector
; --------------------

k_mag = (2.0*!pi/c) * header.freqs	; k for each freq in header
kd = fltarr(2,header.nfreqs)
for pairx=0,1 do $			; kd for each freq and each antenna pair
   kd(pairx,0) = leaddim1( k_mag * site.separation(pairx) )
temp_xy = d.xphase / kd(*,d.x.f)	; sin(theta_x,y), dims are (2,NSOURCES)
;print, 'sin = ',temp_xy
temp_xy = temp_xy / sqrt(1.0 - temp_xy*temp_xy)		; tan(theta_x,y)
;print, 'tan = ',temp_xy
kz = - k_mag(d.x.f) / sqrt(1+ total(temp_xy*temp_xy,1))	; - for returning echo
sources.k(0) = kz * temp_xy(0,*)			; kz * tan(theta_x)
sources.k(1) = kz * temp_xy(1,*)			; kz * tan(theta_y)
sources.k(2) = kz
;stop
form_result:

return, create_struct( $
   'HEADER_EXISTS', cadi.header_exists, $
   'NFREBINS', cadi.nfrebins, $
   'NDATA', nsources, $
   'DATATYPE', 'KVEC', $
   'HEADER', cadi.header, $
   'FREBINS', cadi.frebins, $
   'DATA', sources)

form_nullresult:

sources = -1L
nsources = 0L
goto, form_result

end
;+
; NAME:
;	CADI_XAP_VETAOA
; PURPOSE:
;	Reject CADI sources with cross-phases (angle of arrival) outside limits.
; CALLING SEQUENCE:
;	RESULT = CADI_XAP_VETAOA( CADI_XAP [,/NOALIASAOA] )
; INPUTS:
;	CADI_XAP: Variable of CADI structure with XAP data.
; KEYWORD PARAMETERS:
;	NOALIASAOA: If present and nonzero, the limits on the cross-phases are
;	   set to accept only those sources which lie in the zone around the
;	   zenith of unambiguous source directions.
;	      The default is to accept all sources which lie above the horizon.
; OUTPUTS:
;	Result: Variable of CADI structure with XAP data. Result.DATA is a
;	   subset of CADI_XAP.DATA.
; PROCEDURE:
;
;	Let k be the magnitude of the wave vector, and d the antenna separation.
;	Cross-phase dphi is phase difference between antennas.
;	For a source to be 'above the horizon': |dphi| < | kd |.
;	Sources not satisfying this condition are always rejected.
;	For a source to be in the zone of unambiguous angle-of-arrival (AOA),
;	it must be above the aliased horizon: |dphi| < | kd - 2*pi |.
;
;	To allow a little room for phase errors, the limits on cross-phase for
;	the zone of unambiguous AOA are multiplied by 0.9 before being applied.
;
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 9 Sep 1994: Written by adapting CADI_MD_SOURCES.
;-

function cadi_xap_vetaoa, cadi, noaliasaoa=noaliasaoa

on_error, 2

c = 2.998e8		; m/s
alias_factor = 0.9	; Some allowance for phase errors before accepting
                        ; sources as being in the zone of unambiguous AOA.

if cadi.datatype ne 'XAP' then $
   message, 'Argument does not have XAP (cross-amplitude and phase) data'
nsources = cadi.ndata
if nsources eq 0 then $
   goto, form_nullresult
header = cadi.header
d = cadi.data
site = cadi_site(header.site, header.time)

; Find limit on cross-phase

k_mag = (2.0*!pi/c) * header.freqs	; k for each freq in header
kd = fltarr(header.nfreqs,2)
for pairx=0,1 do $			; kd for each freq and each antenna pair
   kd(0,pairx) = k_mag * site.separation(pairx)
if keyword_set(noaliasaoa) then $		; Horizon & aliased horizon
   phase_limit = kd < alias_factor*abs(kd - 2.0*!pi) $
else $
   phase_limit = kd				; Horizon only

; Reject sources with at least one cross-phase outside limit

nxapsources = nsources
wgood = where( (abs(d.xphase(0)) le phase_limit(d.x.f, 0) ) and $
               (abs(d.xphase(1)) le phase_limit(d.x.f, 1) ) ,nsources)
;%% IDL requires different syntaxes for DIM(SOURCES) EQ/GT 1
;%% Until I think of a tidy workaround, don't process NSOURCES EQ 1
;%% 940909 Try to accept NSOURCES eq 1 %% if ngood le 1 then $	;%% Added 940810
if nsources eq 0 then $
   goto, form_nullresult	; All bins rejected
if nsources lt nxapsources then $
   d = d(wgood)

form_result:

return, create_struct( $
   'HEADER_EXISTS', cadi.header_exists, $
   'NFREBINS', cadi.nfrebins, $
   'NDATA', nsources, $
   'DATATYPE', cadi.datatype, $
   'HEADER', cadi.header, $
   'FREBINS', cadi.frebins, $
   'DATA', d)

form_nullresult:

d = -1L
nsources = 0L
goto, form_result

end
