;+ 
; NAME: 
;       CADI_CONCAT 
; PURPOSE: 
;       Concatenate two CADI structures (in time). 
; CALLING SEQUENCE: 
;       Result = CADI_CONCAT( CADI1, CADI2) 
; INPUTS: 
;       CADI1, CADI2: Two variables of CADI structure. Both are mandatory. 
;          Each, if it has DATA elements, must have a .DATA.X.T field. 
;          The times CADI1.HEADER.TIMES and CADI2.HEADER.TIMES should each be 
;          in sequence, the two should not overlap, and the first element of 
;          CADI2.HEADER.TIMES should exceed the last of CADI1.HEADER.TIMES. 
; OUTPUTS: 
;       Result: A variable of CADI structure. 
;          Result.HEADER.TIMES is [ CADI1.HEADER.TIMES, CADI2.HEADER.TIMES ]. 
;          Result.DATA is [ CADI1.DATA, CADI2.DATA ], with Result.DATA.X.T 
;          modified to point to the appropriate elements of Result.HEADER.TIMES. 
; RESTRICTIONS: 
;       It is assumed without checking that corresponding fields in the two  
;       .HEADERs are the same, except for those relating to times. 
; 
;       %% CADIn.FREBINS not yet concatenated. 
; 
; MODIFICATION HISTORY: 
;       Ian Grant, UWO, 29 Jul 1994. 
;       GRANT, 3 Feb 1995: Ensured correct behaviour if one or both arguments 
;          are null. 
;- 
 
function cadi_concat, cadi1, cadi2 
 
on_error, 2 
 
if not cadi1.header_exists then return, cadi2   ; In case CADI1 or both are null 
if not cadi2.header_exists then return, cadi1   ; In case CADI2 is null 
 
; Both arguments are non-null: do the concatenation. 
 
;%% Should check all non time fields of H1 and H2 are the same; could do this in  
; a loop with tag index. Here we just take the values from H1. 
 
h1 = cadi1.header 
h2 = cadi2.header 
d1 = cadi1.data 
d2 = cadi2.data 
 
header_out = { $ 
         nantennas:     h1.nantennas, $ 
         ndops:         h1.ndops, $ 
         nheights:      h1.nheights, $ 
         ntimes:        h1.ntimes + h2.ntimes, $        ; New 
         nfreqs:        h1.nfreqs, $ 
         site:          h1.site, $ 
         time:          h1.time, $ 
         filetype:      h1.filetype, $ 
         minheight:     h1.minheight, $ 
         maxheight:     h1.maxheight, $ 
         pps:           h1.pps, $ 
         npulses_avgd:  h1.npulses_avgd, $ 
         base_thr100:   h1.base_thr100, $ 
         noise_thr100:  h1.noise_thr100, $ 
         min_dop_forsave:h1.min_dop_forsave, $ 
         dtime:         long(h1.dtime), $               ; New 
         gain_control:  h1.gain_control, $ 
         sig_process:   h1.sig_process, $ 
         dops:          h1.dops, $ 
         heights:       h1.heights, $ 
         freqs:         h1.freqs, $ 
         times:         [ h1.times, h2.times ], $ 
         dir:           h1.dir, $ 
         file_ext:      h1.file_ext}            ; New 
 
n1 = cadi1.ndata 
n2 = cadi2.ndata 
 
if n1 eq 0 then $ 
   d = d2 $ 
else if n2 eq 0 then $ 
   d = d1 $ 
else $ 
   d = [d1, d2] 
 
if n2 gt 0 then $ 
   d(n1:*).x.t = d2.x.t + h1.ntimes 
 
return, create_struct( $ 
   'HEADER_EXISTS', !true, $ 
   'NFREBINS', 0L, $            ;%% Not programmed yet 
   'NDATA', n1+n2, $ 
   'DATATYPE', cadi1.datatype, $ 
   'HEADER', header_out, $ 
   'FREBINS', -1L, $            ;%% Not programmed yet 
   'DATA', d) 
 
end 
