;+
; NAME:
;	CADI_FITV_FUNC
; PURPOSE:
;	Called by CADI_FIT_VEL to pass data through common to SVDFIT.
; CALLING SEQUENCE:
;	Not intended to be called by the user.
; COMMON BLOCKS:
;	CMN_CADI_SVD
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 9 Sep 1994. Written as part of CADI_FIT_VEL.PRO.
;	GRANT, 13 Jan 1995: Extracted from CADI_FIT_VEL.PRO to create a 
;	   separate module.
;-

function cadi_fitv_func, i_list, m

on_error, 2
common cmn_cadi_svd, a

return, a
end
;+
; NAME:
;	CADI_FIT_VEL
; PURPOSE:
;	Fit a velocity vector to a set of CADI sources.
; CALLING SEQUENCE:
;	Result = CADI_FIT_VEL( DF, K )
; INPUTS:
;	DF = Vector with NSOURCES elements. Doppler shift of each source.
;	K = 2D array with dimensions (3,NSOURCES). K-vectors (unnormalized) of 
;	   the returning echo from each source. The three components are
;	   (kx,ky,kz) where X is east, Y is north, Z is vertically upwards.
; KEYWORD PARAMETERS:
;	WEIGHTS = Vector with NSOURCES elements. Weight of each source in 
;	   least-squares fit. Default is equal weighting.
;	LIST: If present and nonzero, then print results of the fit.
;	SINGULAR = Number of singular values returned by SVDFIT (usually zero).
;	CHISQ = Chi-square statistic returned by SVDFIT.
;	STDDEV = The square root of the variance returned by SVDFIT.
; OUTPUTS:
;	Result = Three-dimensional velocity vector (Vx,Vy,Vz).
; COMMON BLOCKS:
;	CMN_CADI_SVD
; PROCEDURE:
;	Uses the procedure of Cannon et al., JGR, 96, 1239, 1991. A single 3D 
;	velocity vector fitted by least-squares to the set of sources. The fit
;	is done with IDL's SVDFIT routine, by passing the design matrix
;	(source k-vector) data through common to the function CADI_FITV_FUNC.
;	Actually, GRANT's SVDFIT_IFG is used to allow execution to continue if 
;	singular values are found.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 9 Sep 1994. Written by adapting CADI_VEL_FIT_3D.
;	GRANT, UWO, 13 Jan 1995: Made CADI_FITV_FUNC a separate module.
;-

function cadi_fit_vel, df, k, weights=weights, $
   list=list, singular=sing, chisq=chisq, stddev=stddev

on_error, 2
common cmn_cadi_svd, a

nsources = n_elements(df)
size_k = size(k)
if (size_k(0) ne 2) or (size_k(1) ne 3) or (size_k(2) ne nsources) then $
   message, 'Dimensions of K must be (3,NSOURCES)'
if not keyword_set(weights) then $
   weights = replicate(1.0, nsources)
size_weights = size(weights)
if (size_weights(0) ne 1) or (size_weights(1) ne nsources) then $
   message, 'Dimension of WEIGHTS must be (NSOURCES)'

if nsources lt 3 then return, -1L

; Fit 3D vector drift velocity

; X east, Y north, Z up.
; e.g. for X: phas>0 implies scatterer is to east of Rx, so returned wave
; vector points west and kx<0. Wave vector is always down, so kz<0 always.

a = transpose(k) / !pi	; Design matrix A. Dims (N,M).
             		; Will pass A to SVDFIT_IFG through CADI_FITV_FUNC.
x = lindgen(nsources)	; Dummy
y = df			; Doppler shifts of scatterers
m = 3			; Number of drift components
yfit = 0			; Define variable
chisq = 0			; Define variable
sing = 0			; Define variable
var = 0			; Define variable
covar = 0			; Define variable
v = SVDFIT_IFG(x, y, m, yfit=yfit, weight=weights, chisq=chisq, $
   singular=sing, variance=var, covar=covar, funct='cadi_fitv_func')
stddev = sqrt(var)
if keyword_set(list) then begin
   print, 'DRIFT_3D:'
   print, '   Number of singular values = ' + strim(sing)
   print, '   v      = ', v,      format='(a,3f10.2)'
   print, '   stddev = ', stddev, format='(a,3f10.2)'
   print, '   chisq  = ', chisq,  format='(a,3f10.2)'
   endif ; list

return, v
end
