;+
; NAME:
;       CADI_HTV2
; PURPOSE:
;       Plot CADI h't and/or velocity against time.
; CALLING SEQUENCE:
;       CADI_HTV, CADI_HT [,VEL=VEL]
; INPUTS:
;       CADI_HT = Variable of CADI structure which will supply the h't
;          information, and also ancillary information such as date, site and
;          sounding frequencies. CADI_HT.DATA need have only the X field.
;          Mandatory.
; KEYWORD PARAMETERS:
;       VEL = Vector of CADI velocity structures. Must be supplied if
;          PANELS specifies velocity parameters.
;       TIMERANGE = Two-element vector of CANOPUS times specifying start and
;          end time of plot.
;       PANELS = Scalar string with one character for each panel to be plotted.
;          Characters may be any combination of the following:
;             f   h't (specify frequency with FREQX_LIST)
;             A   Azimuth of horizontal component of velocity
;             H   Magnitude of horizontal component of velocity
;               X
;               Y
;               S
;               D
;             V   Vertical component of velocity
;             N   Number of sources in the velocity fit.
;             a   Error in azimuth of horizontal component of velocity
;             h   Error in magnitude of horizontal component of velocity
;             v   Error in vertical component of velocity
;       FREQX_LIST = Vector of indices into CADI_HT.HEADER.FREQS which
;          indicate which frequency to display in each h't panel specified by
;          PANELS. FREQX_LIST must have exactly one element for each 'f' in
;          PANELS.
;       ERROR: Integer scalar returned as nonzero if an error occurs.
;       INFORM: If present and nonzero, progress messages are printed.
;       HOR:       Forces y-range of 'H' panel to be [0, HOR].
;       VER:       Forces y-range of 'V' panel to be [-VER, VER].
;       EAZI:      Forces y-range of 'a' panel to be [0, EAZI].
;       EHOR:      Forces y-range of 'h' panel to be [0, EHOR].
;       EVER:      Forces y-range of 'v' panel to be [0, EVER].
;       NSOURCES:  Forces y-range of 'N' panel to be [0, NSOURCES].
;       POS:    4-element float vector of normalized coordinates specifying the
;               rectangle (x0,y0,x1,y1) on the display area in which to plot.
;               Default is [0.12, 0.1, 0.97, 0.95].
; RESTRICTIONS:
;       %% At present, all data might be plotted, rather than that within
;          TIMERANGE. Even past hour boundaries!!
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 19 Sep 1994: Adapted from CADI_MD_HTV.PRO.
;       GRANT, 2 Nov 1994: Added call to CADI_REDUCETIME to speed FHT h't plot.
;       GRANT, 9 Nov 1994: Added POS keyword.
;       GRANT, 15 Nov 1994: Put 'Err!c' into ytitle of each error trace.
;       GRANT, 11 Jan 1995: Changed CADI_HT from keyword to plain parameter.
;       GRANT, 8 Feb 1995: Added handling of null CADI_HT or VEL.
;       JMacD, 13 Apr 1995: Added pcl plotting
;       JMacD, 16 July 1996: added datadump
;       JMacD, 23 Sept 1997: added extended height
;		JMacD, 11 Dec 1997: datadump modified to dump xyz for Fortaleza
;		JMacD, 16 June 1999: datadump corrected to dump v(i).(4) for azimuth instead of y(i)
;		JMacD, 13 June 2001: dumph keyword to dump ht data
;       JMacD,  9 Oct 2002: added plotting of sunward and dawn dusk horizontal velocities
;       JMacD, 3 April 2004: modified for widgets, added colour keyword, changed dumpfile to dumpv
;		JMacD  18 April 2004 added minheight
;-

pro cadi_htv2, cadi_ht, cadi_ht2, vel1 = v_in, vel2 = v_in2, timerange=timerange, $
   panels=panels_str, freqx_list=htfreqx_list, maxheight = maxheight, minheight = minheight,$
   error=error, inform=inform, $
   hor=hor, ver=ver, eazi=eazi, ehor=ehor, ever=ever, nsources=nsources, $
   pos=pos_regions, dumpv=dumpv, overplot = overplot, dumph = dumph, colour = colour

;if keyword_set(colour) then begin
;  !p.multi = [0,1,2]
;  plot, [0],[0],/nodata, position = [0.05, 0.05,0.93,0.95]
;endif else begin
;  !p.multi = [0,1,1]
;  plot, [0],[0], /nodata, position = [0.05,0.05,0.95,0.95]
;endelse

;%% VEL needs HEADER here only for:
;    - SITE for title
;    - Vel freq for title
;    - Default timerange

;%% Check h't, velocity and header info all come from the same site?

;%% Ensure only that subset of h't and vel within TIMERANGE are plotted.

on_error, 0 ;2
erase  ;clear screen of everything


wild_char = '*'
max_value = 9.0e8       ; 900 MHz
min_azi = 0.0
;stop
if not keyword_set(panels_str) then panels_str = 'AHVNahv'
panels_str = 'g'+ panels_str
;print, 'panel str = ',panels_str
if not keyword_set(hor) then hor = 2000.0
if not keyword_set(ver) then ver = 500.0
if not keyword_set(eazi) then eazi = 50.0
if not keyword_set(ehor) then ehor = 90.0
if not keyword_set(ever) then ever = 50.0
if not keyword_set(nsources) then nsources = 100 ;200
if not keyword_set(pos_regions) then begin
   if keyword_set(colour) then pos_regions = [0.10, 0.10, 0.93, 0.95] else  pos_regions = [0.12, 0.1, 0.97, 0.95]
endif

if n_params() lt 1 then $
   message, 'Must supply argument CADI_HT'
if not cadi_ht.header_exists then $
   message, 'CADI_HT.HEADER does not exist'

;%% ht = cadi_ht.data
h = cadi_ht.header
h2 = cadi_ht2.header
stationnumber = 1

if not keyword_set(minheight) then pltminheight = h.minheight else begin
 if (minheight eq 1) then pltminheight = 0 else pltminheight = minheight ;don't get keyword_set if minheight eq 0 so use 1 and adjust
endelse
if not keyword_set(maxheight) then maxheight = h.maxheight

; Find TIMERANGE, REFTIME for time axis, PLOT_TIMERANGE
if not keyword_set(timerange) then $
   timerange = canmidnight(h.time) + [0,24*3600L]
if timerange(1) - timerange(0) le 6*3600 then $
   reftime = canmidnight(timerange(0)) $
else $
   reftime = canjan0(timerange(0))
plot_timerange = timerange - reftime
;print, 'plot timerange = ', plot_timerange
;stop
nodaylabels = (timerange(1) le (canmidnight(timerange(0)) + (24*3600L)))

; In PANEL_STU, TAGX is index to frequency for h't, otherwise index of tag in V
; structure.

dummy = {panel_stu, $
   ytitle:'', code:' ', type:'', tagx:0L, tagname:'', yrellen:0.0, yrange:[0.0,0.0]}

panel_options = [ $
   ;{panel_stu, "Height!c(km)",         'f', 'HT',  0,  '',        4, [0,510]}, $
   {panel_stu, "Group Range!c(km)",         'g', 'HT',  0,  '',        4, [pltminheight,maxheight]}, $
   {panel_stu, "Group Range!c(km)",         'f', 'HT',  0,  '',        4, [pltminheight,maxheight]}, $
   {panel_stu, 'V_azi!c(deg)',         'A', 'DEG', 0, 'AZI',      4, [min_azi,min_azi+360]}, $
   {panel_stu, 'V_hor!c(m/s)',         'H', '',    0, 'HOR',      4, [0,hor]}, $
   {panel_stu, 'V_ver!c(m/s)',         'V', '',    0, 'VER',      4, [-ver,ver]}, $
   {panel_stu, 'V_horEW!c(m/s)',       'X', '',    0, 'HOREW',    4, [-hor,hor]}, $
   {panel_stu, 'V_horNS!c(m/s)',       'Y', '',    0, 'HORNS',    4, [-hor,hor]}, $
   {panel_stu, 'V_sunward!c(m/s)',         'S', '',    0, 'HORSUN', 4, [-hor,hor]}, $
   {panel_stu, 'V_duskward!c(m/s)',        'D', '',    0, 'HORDUSK', 4, [-hor,hor]}, $
   {panel_stu, 'Number!cof!csources',  'N', 'NOS',    0, 'NSOURCES', 1, [0,nsources]}, $
   {panel_stu, 'Err!cAzi',             'a', '',    0, 'AZI_ERR',  1, [0,eazi]}, $
   {panel_stu, 'Err!cHor',             'h', '',    0, 'HOR_ERR',  1, [0,ehor]}, $
   {panel_stu, 'Err!cVer',             'v', '',    0, 'VER_ERR',  1, [0,ever]} $
      ]

npanels = strlen(panels_str)
panels = replicate({panel_stu}, npanels)

for panelx=0,npanels-1 do begin
   panel_code = strmid(panels_str, panelx, 1)
   panel_optionx = (where(panel_code eq panel_options.code)) (0)
   if panel_optionx lt 0 then begin
      message, /inform, 'Unrecognized code in PANELS keyword'
      goto, error_return
   endif
   panels(panelx) = panel_options(panel_optionx)
endfor ;panelx = 0

;stop
htpanelx_list = where(panels.type eq 'HT', npanels_ht)
vpanelx_list = where(panels.type ne 'HT', npanels_v)

if npanels_ht gt 0 then begin
   ;%% if not keyword_set(cadi_ht) then begin
   ;%%    message, /inform, $
   ;%%       'Keyword PANELS_STR='+panels_str+ $
   ;%%       ' requires keyword CADI_HT must be given'
   ;%%    goto, error_return
   ;%%    endif
   if not keyword_set(htfreqx_list) then $
      htfreqx_list = reverse(lindgen(npanels_ht))

   ; Check h't frequency list is valid.

         if n_elements(htfreqx_list) ne npanels_ht-1 then begin
      message, /inform, $
         'Number of HTFREQX_LIST elements must equal number of "f" panels'
      goto, error_return
      endif

   if (howmany(htfreqx_list gt h.nfreqs-1) gt 0) or $
      (howmany(htfreqx_list lt 0) gt 0) then begin
      message, /inform, $
        'HTFREQX_LIST elements are out of range; max allowed is '+strim(h.nfreqs-1)
      goto, error_return
      endif

   ; Store frequency list in the TAG field of the h't PANELS.

   if npanels_ht-1 gt 1 then $
      panels(where(panels.type eq 'HT')).tagx = htfreqx_list $  ; LHS a vector
   else $
      panels(where(panels.type eq 'HT')).tagx = htfreqx_list(0) ; LHS a scalar
   endif

if npanels_v gt 0 then begin
   if not keyword_set(v_in) then begin
      message, /inform, $
         'Keyword PANELS_STR='+panels_str+' requires keyword VEL must be given'
      goto, error_return
      endif
   v = v_in
   v2 = v_in2
   ;stop
   if (size(v_in)) (0) eq 0 then begin

      ; No elements in velocity argument, so create two missing velocities to
      ; avoid problems.

     dummy = cadi_kvec2vel(missing=v_missing)
     v = replicate(v_missing, 2)
     v_missing.time = 0L  ; timerange
   endif
   v_tagnames = tag_names(v)

   for panelxx=0,npanels_v-1 do begin
      panelx = vpanelx_list(panelxx)
      ; Set tag numbers from the velocity argument itself, in case the
      ; velocity structure changes in future.
      panels(panelx).tagx = (where(v_tagnames eq panels(panelx).tagname)) (0)
   endfor
endif ; npanels_v gt 0

; Temporary list of Y norm coords of panel horizontal edges.
; First panel (panelx=0) is at the top.

pos = pos_regions(3) - (pos_regions(3) - pos_regions(1)) * $
   (cumulative_sum([0, panels.yrellen]) / total(panels.yrellen))
; Each row will be the position of the panel corners in norm coordinates.
pos_list = transpose( [ $
         [replicate(pos_regions(0), npanels)], $
         [pos(1:npanels)], $
         [replicate(pos_regions(2), npanels)], $
         [pos(0:npanels-1)] $
            ] )

psav = !p
xsav = !x
ysav = !y
pticklen = !p.ticklen

if npanels_v gt 0 then begin

   ; Nonmissing drift points

   w_nonmiss = where(v.azi lt 400.0, nonmiss_count) &&

   ; Put drift freq into string

   if nonmiss_count gt 0 then begin
      min_fx = min(v(w_nonmiss).fxrange(0))
      max_fx = max(v(w_nonmiss).fxrange(1))
      if min_fx eq max_fx then $
         vfreq_str = strim(h.freqs(min_fx) / 1e6, format='(f6.3)') + ' MHz' $
      else $
         vfreq_str = 'Multi-freq'
      endif $
   else $
      vfreq_str = '0.000' + ' MHz'

   endif ; npanels_v gt 0

; Start plot

erase

; Set up to plot h't images.

if npanels_ht gt 0 then begin

   ; Set up device for images

if keyword_set(colour) then begin
      back_color = 255b 	;white
      !p.background = 255b
      fore_color = 0b		;black
      !p.color = 0b
      charsize = 1.5
      stn1color = 40
      stn2color = 140
      stnbothcolor = 100
endif else begin
   if !d.name eq 'PS' then begin
      ;device, bits_per_pixel=1 ;%% Better without DEVICE call in this procedure?
      device, bits_per_pixel=8 ;%% Better without DEVICE call in this procedure?
      back_color = 0b ;255b ; White
      fore_color = 255b ;0b   ; Black
      charsize = 1.5
   endif else $
   if !d.name eq 'PCL' then begin
      back_color = 255b ;0b  ; White
      fore_color = 0b ;255b  ; Black
      charsize = 1.5
   endif else	begin	;probably just windows
      back_color = 0b   ; White
      fore_color = 255b ; Black
      charsize = 1.5
   endelse
endelse ;not color
erase
   ; Determine default pixel height DH.

   dh = median(difference( h.heights ))         ; Typical height step.

   endif ; npanels_ht gt 0

; Plot each panel
npanelx = npanels -1

;panels have been set up so plot data
for panelx=0,npanelx do begin
   panel = panels(panelx)
   pos = pos_list(*,panelx)

   if panelx eq 0 then begin
      title_ht = ''
      title_and = ''
      title_v = ''
      if npanels_ht gt 0 then $
         title_ht = " h't"
      if npanels_ht gt 0 and npanels_v gt 0 then $
         title_and = ' and'
      if npanels_v gt 0 then $
         title_v = ' drift (' + vfreq_str +')'
      title = 'CADI' + title_ht + title_and + title_v + '    ' + $
         '2 stations  ' + strcantime(timerange(0),/date)
      endif $
   else $
      title = ''




   case panel.type of
      'HT' : begin

;caseHT1:
       ;stop
         ; Define axes of subplot without drawing

      plot, /noerase, /nodata, [0], [0], pos=pos, charsize = charsize,$
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ystyle=5
;stop
         ;oplot, [71700,71700],[0,510],linestyle=2
         ;oplot, [73500,73500],[0,510],linestyle=2
         ;oplot, [75300,75300],[0,510],linestyle=2
   ; Find pixels for this frequency ;%% and time range

         ;%% This SUB line takes ~80% of time before image plotting starts
         ;%% ht_onef = cadi_sub(ht, frange=[panel.tagx,panel.tagx], wcount)
         cadi_ht_onef = cadi_sub_struct(cadi_ht, frange=[panel.tagx,panel.tagx])
         cadi_ht_onef2 = cadi_sub_struct(cadi_ht2, frange=[panel.tagx,panel.tagx])

            ;find the data in the selected height range
            ;stop
            ; select = where(cadi_ht_onef.data.x.h lt maxheight/3)
            ;cadi_ht_onef = cadi_ht_onef.data(select)
         ;%% if wcount gt 0 then begin
         if cadi_ht_onef.ndata gt 0 then begin

;stop
            ; Some pixels are set for this frequency and hour.

            ; If FHT, keep only the times with data

            if h.file_ext eq 'FHT' then $
               cadi_ht_onef = cadi_reducetime(cadi_ht_onef)
               cadi_ht_onef2 = cadi_reducetime(cadi_ht_onef2)

            ;%% Extract HEADER and DATA fields for this code to plot the image

            ;%% h_onef = cadi_ht_onef.header
            ;%% ht_onef = cadi_ht_onef.data

            ; Determine default pixel width DT.

            if h.file_ext eq 'FHT' then $
               dt = h.nfreqs * h.dtime $
            else $
               dt = h.dtime

            ; Find smallest rectangle on t-h plane which includes all set pixels.

            ; %% This was patched together from existing bits of code. It would
            ; probably be more efficient of space to construct the image only for
            ; each inter-jump interval at a time, instead of the whole Z at once
            ; as done here. The MIN_XT etc. stuff is not strictly necessary here
            ; since something similar is done inside CADI_TVFT, but I leave it
            ; here to reduce the size of the complete Z.

           ; min_xt = min(cadi_ht_onef.data.x.t, max=max_xt)
           ; min_xh = min(cadi_ht_onef.data.x.h, max=max_xh)
           ; tdim = max_xt - min_xt + 1
           ; min_xt2 = min(cadi_ht_onef2.data.x.t, max=max_xt2)
           ; min_xh2 = min(cadi_ht_onef2.data.x.h, max=max_xh2)
           ; tdim2 = max_xt2 - min_xt2 + 1
           ;print, min_xt,max_xt,plot_timerange, dt
            min_xt = 0
            max_xt = (plot_timerange(1)-plot_timerange(0))/dt
            min_xt2 = min_xt
            max_xt2 = max_xt
           ;print, min_xt,max_xt
            tdim = max_xt - min_xt
            tdim2 = tdim
            tcor = max_xt - 1 - max(cadi_ht_onef.data.x.t)
            tcor2 = max_xt - 1 - max(cadi_ht_onef2.data.x.t)
; stop
            if keyword_set(maxheight) then max_xh = fix(float(maxheight)/3)-1	;adjust heightrange if maxheight specified
            if keyword_set(minheight) then min_xh = fix(float(pltminheight)/3) 	;adjust heightrange if minheight specified
           ; print, 'heightrange = ', min_xh, max_xh
            max_xh = fix(panel.yrange(1)/3)
            min_xh = fix(panel.yrange(0)/3)
            max_xh2 = max_xh
            min_xh2 = min_xh
         ;  print, max_xh,min_xh
    		hdim = max_xh - min_xh + 1
    		hdim2 = max_xh2 -min_xh2 + 1
         ;  print, 'hdim =', hdim
;stop
;%% From SUB line to here takes ~10% of time before image plotting starts

            ; Create an image of this smallest rectangle and plot it

            z = 0             ; Deallocate memory
            ;z2 = 0
            z = make_array( tdim, hdim, /byte,value = back_color)
            ;z2 = make_array(tdim2, hdim2, /byte, value = back_color)
            ;if (size(z2))(0) ne 2 then $
               ; Ensure Z is 2D even if one or both dimensions are 1
             ; z2 = reform( temporary(z2), tdim, hdim )
			if (size(z))(0) ne 2 then $
               ; Ensure Z is 2D even if one or both dimensions are 1
              z = reform( temporary(z), tdim, hdim )
		if keyword_set(colour) then begin
            if stationnumber eq 1 then begin
			  for p = 0L,long(cadi_ht_onef.ndata-1) do begin
			  ;stop
			    xpos = cadi_ht_onef.data(p).x.t + tcor;-min_xt
			    ypos = cadi_ht_onef.data(p).x.h-min_xh
			    rept = fix(cadi_ht_onef.data(p).IQ(0,0))
			    if (rept gt 127) then rept = rept - 256
			    impt = fix(cadi_ht_onef.data(p).IQ(1,0));-128
			    if (impt gt 127) then impt = impt - 256
;print, 're,im = ', rept,impt
			    pow = rept^2 + impt^2
			    powdb = byte(7*10*alog10(pow))
			    if (powdb gt 254) then powdb = 254
                if (ypos ge 0)and(ypos lt hdim) then begin
                   if ((z(xpos,ypos) eq 255) or (powdb gt 255 - z(xpos,ypos))) then z(xpos,ypos) = stn1color; blue ;255 - powdb
                endif
			   endfor
			  endif
 ;stop
            ; tvxy, z,$
            ;   cadi_ht_onef.header.times(min_xt:max_xt-1)-reftime, $
            ;     cadi_ht_onef.header.heights(min_xh:max_xh), $
            ;       dx=dt, dy=dh
       ; stop
				;repeat for 2nd station
		     if stationnumber eq 2 then begin
			  for p = 0L,long(cadi_ht_onef2.ndata-1) do begin
			  ;stop
			;    xpos = cadi_ht_onef2.data(p).x.t-min_xt2
			    ;print, 'xpos = ', xpos
			    xpos = cadi_ht_onef2.data(p).x.t + tcor2;-min_xt
			    ypos = cadi_ht_onef2.data(p).x.h-min_xh2
			    rept = fix(cadi_ht_onef2.data(p).IQ(0,0))
			    if (rept gt 127) then rept = rept - 256
			    impt = fix(cadi_ht_onef2.data(p).IQ(1,0));-128
			    if (impt gt 127) then impt = impt - 256
;print, 're,im = ', rept,impt
			    pow = rept^2 + impt^2
			    powdb = byte(7*10*alog10(pow))
			    if (powdb gt 254) then powdb = 254
                if (ypos ge 0)and(ypos lt hdim2) then begin
                   ;if ((z(xpos,ypos) eq 255) or (powdb gt 255 - z(xpos,ypos))) then z(xpos,ypos) = 200; yellow ;255 - powdb
                   if z(xpos,ypos) eq stn1color then z(xpos,ypos) = stnbothcolor else if z(xpos,ypos) ne stnbothcolor then z(xpos,ypos) = stn2color
                endif
			   endfor
			  endif
   ;stop

          trange = lindgen(tdim)*dt + plot_timerange(0)
             tvxy, z,$
               ;cadi_ht_onef.header.times(min_xt2:max_xt2-1)-reftime, $
                 trange, $
                 cadi_ht_onef.header.heights(min_xh2:max_xh2), $
                  dx=dt, dy=dh
	        endif else begin
	          z( cadi_ht_onef.data.x.t-min_xt, cadi_ht_onef.data.x.h-min_xh ) = fore_color
            ;stop
              cadi_tvft, z, $
               cadi_ht_onef.header.times(min_xt:max_xt)-reftime, $
                 cadi_ht_onef.header.heights(min_xh:max_xh), $
                   dx=dt, dy=dh, fore_color=!p.color, back_color= !p.background
           endelse
           ;stop
		;routine to dump the ht data

        if keyword_set(dumph) then begin
           print, "making data file"
           tstr = strcantime(timerange(0),/date)
           dumpfilename = 'c:\cdata\' + tstr + '.th'
           openw, 1, dumpfilename;, /append         put back append if wanted
			 select = where(z gt 0, nmbr)
             nt = n_elements(z(*,0))
			 nh = n_elements(z(0,*))
			 ;print, nt, nh
			 datatimearr = cadi_ht_onef.header.times(min_xt:max_xt)
			 rtime = 86400L*floor(datatimearr(0)/86400)
			 dataheightarr = cadi_ht_onef.header.heights(min_xh:max_xh)
			 printf, 1, h.freqs(panel.tagx)
             printf, 1, nmbr
             for i = 0,nt-1 do begin
             for j = 0,nh-1 do begin
             if z(i,j) ne back_color then begin
;print, i,j,z(i,j)
               printf, 1,datatimearr(i)-rtime, dataheightarr(j)
             endif
             endfor
             endfor
           close, 1
       endif ;dumph
;stop
            endif ; wcount gt 0

         ; Plot axes only for h't. Image has been plotted already.
;stop
         ;htfreq_str = strim(h.freqs(panel.tagx) / 1e6, format='(f6.3)') + ' MHz'
        htfreq_str = strim(h.freqs(0) / 1e6, format='(f6.3)') + ' MHz'
         plot, [0], [0], /nodata, $
            /noerase, pos=pos, title=title, charsize = charsize,$
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ytitle=htfreq_str+'!c'+panel.ytitle, ystyle=1
         stationnumber = stationnumber + 1
         end ;case = HT1

     ; 'HT2': begin
     ;   goto, caseHT1
     ;        end

      'DEG': begin
         ; Azimuth

         ; Move angle (azimuth) into range [min_azi, min_azi+360.0]

         y0 = v.(panel.tagx)
         if nonmiss_count gt 0 then $
            y0(w_nonmiss) = posmod(y0(w_nonmiss)-min_azi,360.0)+min_azi

         ; Prepare to gracefully plot 360 degree jumps

         wrap_preplot, v.time, y0, min_azi, min_azi+360.0, $
            t, y, missing=1.1*max_value

         plot, [t-reftime], [y], $
            /noerase, max_value=max_value, pos=pos, psym=-3, title=title, $
            xrange=plot_timerange, xstyle=5, charsize = charsize,$
            yrange=panel.yrange, ytitle=panel.ytitle, ystyle=1, $
            yticks=4, yminor=9, color = stn1color

         y02 = v2.(panel.tagx)
         if nonmiss_count gt 0 then $
            y02(w_nonmiss) = posmod(y02(w_nonmiss)-min_azi,360.0)+min_azi

         ; Prepare to gracefully plot 360 degree jumps

         wrap_preplot, v2.time, y02, min_azi, min_azi+360.0, $
            t2, y2, missing=1.1*max_value

         oplot, [t2-reftime], [y2], color = stn2color, max_value = max_value


  ;stop
         end
    'NOS': begin
         plot, [v.time-reftime], [v.(panel.tagx)], $
            /noerase, max_value=max_value, pos=pos, psym=-3, title=title, $
            xrange=plot_timerange, xstyle=5, charsize = charsize,$
            yrange=panel.yrange, ytitle=panel.ytitle, ystyle=1, $
            yticks=2, color = stn1color
         end
      else: begin
     ; print, 'tagx =',panel.tagx

        if keyword_set(dumpv) then begin					;make datafile from velocities
           print, "making data file from #" , panel.tagx
           tstr = strcantime(timerange(0),/date)
           dumpfilename = 'c:\cdata\' + tstr + '.thv'
           openw, 1, dumpfilename  ;, /append
		       ;openw, 1, 'a:output.dat'
if (panels_str eq 'fAH') or (panels_str eq 'fAHV') or (panels_str eq 'fAHVN') or (panels_str eq 'fAHVNahv') then begin
		       select = where(v.azi gt 0.0 and v.azi lt 370.0)	;get rid of bad data
		       v = v(select)
		     endif else begin
		       select = where(v.horew gt -5000.0 and v.horew lt 5000.0)
		       v = v(select)
		     endelse
             n = n_elements(v)
             reftime0 = canmidnight(timerange(0))
             printf, 1, n									;write number of lines in file

             for i = 0,n-1 do begin
                if panels_str eq 'fAH' then printf, 1, format = '(i10,2f10.1)', v(i).time - reftime0, v(i).azi,$
                                                 v(i).hor
               if panels_str eq 'fXY' then printf, 1, format = '(i10,2f10.1)', v(i).time - reftime0, v(i).horew,$
                                                 v(i).horns
               if panels_str eq 'fAHV' then printf, 1, format = '(i10,3f10.1)', v(i).time - reftime0, v(i).azi,$
                                                 v(i).hor, v(i).ver
               if panels_str eq 'fXYV' then printf, 1, format = '(i10,3f10.1)', v(i).time - reftime0, v(i).horew,$
                                                 v(i).horns, v(i).ver
               if panels_str eq 'fAHVN' then printf, 1, format = '(i10,3f10.1,i10)', v(i).time - reftime0, v(i).azi,$
                                                 v(i).hor, v(i).ver,v(i).nsources
               if panels_str eq 'fXYVN' then printf, 1, format = '(i10,3f10.1,i10)', v(i).time - reftime0, v(i).horew,$
                                                 v(i).horns, v(i).ver,v(i).nsources
               if panels_str eq 'fAHVNahv' then printf, 1, format = '(i10,3f10.1,i10,3f10.1)', v(i).time - reftime0,$
                               v(i).azi, v(i).hor, v(i).ver, v(i).nsources, v(i).azi_err, v(i).hor_err, v(i).ver_err
               if panels_str eq 'fXYVNahv' then printf, 1, format = '(i10,3f10.1,i10,3f10.1)', v(i).time - reftime0,$
                               v(i).horew, v(i).horns, v(i).ver, v(i).nsources, v(i).azi_err, v(i).hor_err, v(i).ver_err

             endfor
           close, 1
         endif
         plot, [v.time-reftime], [v.(panel.tagx)], $
            /noerase, max_value=max_value, pos=pos, psym=-3, title=title, $
            xrange=plot_timerange, xstyle=5, charsize = charsize,$
            yrange=panel.yrange, ytitle=panel.ytitle, ystyle=1, color = stn1color
           oplot, [v2.time-reftime], [v2.(panel.tagx)],color = stn2color,max_value = max_value
           oplot, plot_timerange, [0,0]

         end
      endcase

   ; Plot timeaxes

   top = (panelx eq 0)
   nolabels = (panelx ne npanels-1)
   if nolabels then $
      xtitle = ' ' $
   else $
      xtitle = ''       ; Use default
   !p.ticklen = pticklen / (pos(3) - pos(1))
   !x.charsize = charsize*0.75   ; Compensate CHARSIZE=0.8 in TIMESCALE
   timescale, plot_timerange(0), plot_timerange(1), title=xtitle, charsize = charsize, $
      top=top, nolabels=nolabels, tic_factor=-1, nodaylabels=nodaylabels
   !p.ticklen = pticklen

endfor ; panelx




;if color then put on db scale
;if keyword_set(colour) then begin
;  yticklabels = ['0','6','12','18','24','30','36']
;  plot, /noerase, [0,1],[0,1],/nodata, yrange = [0,256], ystyle = 1, xrange = [0,1],charsize = 0.75*charsize,$
;     position = [0.97,0.18,0.99,0.85], xstyle = 4, yticks = 6, ytickname = yticklabels, title = 'dB'
;  for c = 0,255 do oplot, [0,1],[c,c],color = 255 - c, thick = 3
;endif

plot, /noerase, [0,1],[0,0], thick = 20, pos = [0.97,0.88,0.99,0.90], color = stn1color, xstyle = 4, ystyle = 4, $
    title = h.site
;plot, /noerase, [0,1],[0,0], thick = 20, pos = [0.97,0.78,0.99,0.80], color = stnbothcolor, xstyle = 4, ystyle = 4, $
 ;   title = 'both'
plot, /noerase, [0,1],[0,0], thick = 20, pos = [0.97,0.68,0.99,0.70], color = stn2color, xstyle = 4, ystyle = 4, $
    title = h2.site


!p = psav
!x = xsav
!y = ysav

error = !false
return

error_return:
error = !true
return

end
