;+ 
; NAME: 
;       CADI_VEL_REGTIM 
; PURPOSE: 
;       Insert missing velocities into a sequence to give roughly regular times. 
; CALLING SEQUENCE: 
;       Result = CADI_VEL_REGTIM( VEL ) 
; INPUTS: 
;       VEL = Vector of CADI velocity structures. 
; OUTPUTS: 
;       Result = Vector of CADI velocity structures which is a superset of VEL. 
;          'Missing' velocity elements are inserted into the vector given by  
;          VEL, so that the times increment roughly regularly. 
; PROCEDURE: 
;       Any time increments exceeding 150% of the median are considered as  
;       gaps. Missing velocity elements are inserted into the gaps. 
; MODIFICATION HISTORY: 
;       Ian Grant, UWO, Oct 1994: Written. 
;- 
 
function cadi_vel_regtim, v_in 
 
on_error, 2 
 
if n_elements(v_in) le 2 then $ 
   return, v_in 
 
nv_in = n_elements(v_in) 
dts = difference(v_in.time) 
dt = round(median(dts)) 
gap_starts = where(dts-dt gt dt/2, ngaps)  ; Which time steps are >50% over nominal? 
if ngaps eq 0 then $ 
   return, v_in 
 
; Find points just before (already done) and just after gaps in original  
; vector, and the lengths of the gaps in the result. 
 
gap_ends = gap_starts + 1 
gap_lens = round(float(dts(gap_starts)) / dt) - 1       ; Number of points to insert in each gap 
 
; Find number of continuous segments of original vector, and their endpoints. 
 
nsegs = ngaps + 1 
seg_starts = [0, gap_ends] 
seg_ends = [gap_starts, nv_in-1] 
 
; Find offset of each segment in result relative to its position in original, 
; and endpoints of segments in result. 
 
seg_offsets = [0, cumulative_sum(gap_lens)] 
newseg_starts = seg_starts + seg_offsets 
newseg_ends = seg_ends + seg_offsets 
 
; Get structure for "missing" element. 
 
dummy = cadi_kvec2vel(missing_vel=v_missing) 
 
; Reserve space for result: length of original data plus total length of gaps. 
 
v = replicate(v_missing, nv_in+seg_offsets(nsegs-1)) 
 
; Move each continous segment of original vector to result, leaving gaps. 
 
for segx=0,nsegs-1 do $ 
   v( newseg_starts(segx) : newseg_ends(segx) ) = $ 
      v_in( seg_starts(segx) : seg_ends(segx) ) 
 
; Set times for result elements in each gap. 
; Interpolate the time differences across the gap, then add each interpolate  
; to the last time before the gap. 
 
for gapx=0,ngaps-1 do $ 
   v( newseg_ends(gapx) + 1 : newseg_ends(gapx) + gap_lens(gapx) ).time = $ 
      v(newseg_ends(gapx)).time + $ 
      round(interpolate( [0.0, float(dts(gap_starts(gapx)))], $ 
      (findgen(gap_lens(gapx))+1) / (gap_lens(gapx)+1) )) 
 
return, v 
end 
