;+
; NAME:
;	ASPECT
; PURPOSE:
;	Returns aspect ratio (X/Y) of device, Plot Region, etc.
; CALLING SEQUENCE:
;	Result = ASPECT( [/DEVICE | /REGION | /POSITION] )
; INPUTS:
;	Non-keyword parameters are ignored.
; KEYWORD PARAMETERS:
;	DEVICE:   If present and nonzero, returns aspect ratio of device area.
;	REGION:   If present and nonzero, returns aspect ratio of Plot Region.
;	POSITION: If present and nonzero, returns aspect ratio of Plot Window.
;	At most one keyword parameter may be set.
;	Default is /DEVICE.
; OUTPUTS:
;	Result = FLOAT scalar. Aspect ratio, given by (X size) / (Y size),
;	  where the sizes are in physical units (e.g. centimeters) on the
;	  output device.
; PROCEDURE:
;	Examines system variables.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 22 Dec 1992: Written in IDL2, by combining
;	   ASPECTD_XONY, ASPECTREG_XONY.
;	GRANT, UWO, 20 Apr 1993: Added POSITION keyword.
;-

function aspect, dummy, device=device, region=region, position=position

on_error, 2
case howmany( [keyword_set(device), keyword_set(region), $
               keyword_set(position)] ) of
   0:    device = !true
   1:    dum = 0
   else: message, 'Must set no more than one of DEVICE, REGION, POSITION'
   end

if keyword_set(device) then $
   return, float(!d.x_vsize) / float(!d.y_vsize)

if keyword_set(region) then $
   if !p.region(0) ne !p.region(2) then $
      ; Plot Region is set
      return, ((!p.region(2)-!p.region(0)) * float(!d.x_vsize)) / $
              ((!p.region(3)-!p.region(1)) * float(!d.y_vsize)) $
   else $
      ; Plot Region is not set: Default Plot Region is Device Area
      return, aspect(/device)

if keyword_set(position) then $
   if !p.position(0) ne !p.position(2) then $
      ; Plot Window is set
      return, ((!p.position(2)-!p.position(0)) * float(!d.x_vsize)) / $
              ((!p.position(3)-!p.position(1)) * float(!d.y_vsize)) $
   else $
      ; Plot Window is not set:
      ; Default Plot Window is set inside Plot Region according to ![XY].MARGIN.
      message, 'Default Plot Window not implemented'	;%%

end
