;+
; NAME:
;	COLORS16
; PURPOSE:
;	Loads 16 colours for color X windows, monochrome PostScript or LJ-250.
; CALLING SEQUENCE:
;	result = COLORS16(image_values)
; OPTIONAL INPUT PARAMETERS:
;	IMAGE_VALUES = Vector or array (e.g. an image) of integer values in the
;	range 0..15, or an integer scalar in the range 1..14.
; KEYWORD PARAMETERS:
;	BACKCOLOR = output element of COLORS for background color of device
;	FORECOLOR = output element of COLORS for foreground color of device
;	RESET: If present and non-zero then color tables are restored to what 
;	   they were before the first call of COLORS16 since the last RESET.
; OUTPUTS:
;	Result:
;	   If no argument: vector of indices COLORS to the 16 color table
;	      entries.
;	   If scalar argument present: COLORS(1+bindgen(image_values)), i.e. the
;	      first IMAGE_VALUES colors from the list, excluding the first.
;	   If non-scalar argument present: COLORS(image_values).
;	   RESETting of color tables sets COLORS = bindgen(16).
; COMMON BLOCKS:
;	CMN_COLORS16
; SIDE EFFECTS:
;	Loads color tables.
; PROCEDURE:
;	On first call, the function calls TVLCT, R, G, B, with R, G, B chosen
;	according to !D.NAME, and creates a vector of indices to the desired 
;	16 entries of the color table. On the first and subsequent calls, the 
;	result is the input with each of its values in the range 0..15 replaced
;	by the respective color table indices.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 22 Aug 1991: Written in IDL2.
;	IFG, 22 Dec 1992: Added ON_ERROR.
;-

function colors16, image_values_in, reset=reset, $
   backcolor=backcolor, forecolor=forecolor

common cmn_colors16, colors16_on, colors16_device, colors, rsav, gsav, bsav, $
   backcolorsav, forecolorsav

on_error, 2
if n_elements(image_values_in) eq 0 then image_values_in = bindgen(16)
if n_elements(colors16_on) eq 0 then colors16_on = !false

image_values = image_values_in
if not isarray(image_values) then $
   ; IMAGE_VALUES is a scalar.
   image_values = 1+bindgen(image_values)

if keyword_set(reset) then begin
   if not colors16_on then $
      message, 'Keyword /RESET invalid when NOT COLORS16_ON'
   tvlct, rsav, gsav, bsav
   colors = bindgen(16)
   colors16_on = !false
   endif $
else begin

if not colors16_on then begin
   ; First call or first since reset.
   ; Actually, we don't need to save and reset color table for devices which 
   ; don't use a color table, e.g. monochrome PostScript; but it has no effect.
   tvlct, /get, rsav, gsav, bsav
   colors16_on = !true
   colors16_device = ''
   endif

if !d.name ne colors16_device then begin
   ; Set up for first or new device
   case !d.name of
      'X': begin

         ; X windows color (simply using LJ color table)

         r = 2.55 * $
            [0, 10,  4,  3,  2,   3, 12, 30, 89, 72, 62, 53, 53, 65, 76, 100]
         g = 2.55 * $
            [0,  4,  4, 10, 22,  26, 38, 53, 83, 41, 21,  8,  5, 19, 45, 100]
         b = 2.55 * $
            [0, 27, 29, 46, 64,  22, 18, 16, 13, 13, 13, 14, 25, 43, 62, 100]
         tvlct, r,g,b
         colors = bindgen(16)
         backcolorsav = colors(0)
         forecolorsav = colors(15)

         ; X windows monochrome

         goto, skip_xmono
         ; Set first 16 entries in color table
         r = [0, 15+16*[indgen(13), 14, 15]]	; Black=0 to white=255
         g = r
         b = r
         tvlct, r,g,b
         colors = bindgen(16)
         backcolorsav = colors(0)
         forecolorsav = colors(15)
         skip_xmono:

         end
      'PS': begin
         ; Monochrome PostScript does not use color tables. So set COLORS to
         ; point to 16 entries in the existing range of 0 to 255.
         colors = reverse( [16*indgen(15), [255]] )	; White=255 to black=0
         backcolorsav = colors(0)
         forecolorsav = colors(15)
         end
      'LJ': begin
         ; Set first 16 entries in color table.
         ;
         ; 14 colours, plus black and white.
         ;
         ;    First 14 colours were selected from Table D-1 (pages D-9 and
         ; D-10), for Lightness 43-57 and Saturation 67-100. Then I removed a
         ; yellow-green (15/6) and an orange-red (10/2) and added two pale
         ; magentas (6/9 and 6/10). This eliminated two regions of the sequence
         ; where the gradation was too fine, and shifted the yellow/yellow-green
         ; boundary to the centre, so negative and positive can be easily
         ; distinguished when zero is at the centre of the scale.
         ;    The colors include, in the following order:
         ; blue, cyan, green, yellow, red, magenta.
         ;    Table 7-6 in LJ-250 Programmer Reference Manual was used to find
         ; RGB equivalents of palette row/column numbers.
         ;
         ; [0/1, 2/10, 0/9, 23/5, 22/7, 18/1, 14/1, 13/1, 12/3, 11/8, 11/4,
         ; 8/7, 6/7, 6/9, 6/10, 25/6]
         ;
         r = 2.55 * $
            [0, 10,  4,  3,  2,   3, 12, 30, 89, 72, 62, 53, 53, 65, 76, 100]
         g = 2.55 * $
            [0,  4,  4, 10, 22,  26, 38, 53, 83, 41, 21,  8,  5, 19, 45, 100]
         b = 2.55 * $
            [0, 27, 29, 46, 64,  22, 18, 16, 13, 13, 13, 14, 25, 43, 62, 100]
         tvlct, r,g,b
         colors = bindgen(16)
         backcolorsav = colors(15)
         forecolorsav = colors(0)
         end
      else: message, 'Procedure not implemented for device ' + !d.name
      endcase
   colors16_device = !d.name
   endif ; (not colors16_on) or (!d.name ne colors16_device)
endelse ; keyword_set(reset)

backcolor = backcolorsav
forecolor = forecolorsav
return, colors(image_values)
end
