;+
; NAME:
;	CURVEFIT_IFG
; PURPOSE:
;	Non-linear least squares fit of function with any number of parameters.
;	Function may be any non-linear function where the partial derivatives
;	are known or can be approximated.
;	This routine differs from CURVEFIT in that flags control whether 
;	parameters are fixed or varying.
; CATEGORY:
;	E2 - Curve and Surface Fitting
; CALLING SEQUENCE:
;	YFIT = CURVEFIT_IFG(X,Y,W,A,IPAR,SIGMAA,ERR)
; INPUTS:
;	X = Row vector of independent variables.
;	Y = Row vector of dependent variable, same length as x.
;	W = Row vector of weights, same length as x and y.
;		For no weighting w(i) = 1.,
;		instrumental weighting w(i) = 1./y(i), etc.
;	A = Vector of nterms length containing the initial estimate
;		for each parameter.  If A is double precision, calculations
;		are performed in double precision, otherwise in single prec.
;	IPAR = Vector of nterms length 0 if parameter is not to be fit
;	                               1 if parameter is to be fit
; OUTPUTS:
;	A = Vector of parameters containing fit.
;	Function result = YFIT = Vector of calculated
;		values.
; OPTIONAL OUTPUT PARAMETERS:
;	Sigmaa = Vector of standard deviations for parameters A.
;	Err = 1 if routine fails to converge, 0 otherwise.
;	   If convergence fails, a warning is printed only if ERR is absent.
; RESTRICTIONS:
;	The function to be fit must be defined and called FUNCT.
;	For an example see FUNCT in the IDL User's Libaray.
;	Call to FUNCT is:
;	FUNCT,X,A,F,PDER
; where:
;	X = Vector of NPOINT independent variables, input.
;	A = Vector of NTERMS function parameters, input.
;	F = Vector of NPOINT values of function, y(i) = funct(x(i)), output.
;	PDER = Array, (NPOINT, NTERMS), of partial derivatives of funct.
;		PDER(I,J) = DErivative of function at ith point with
;		respect to jth parameter.  Optional output parameter.
;		PDER should not be calculated if parameter is not
;		supplied in call (Unless you want to waste some time).
; PROCEDURE:
;	Copied from "CURFIT", least squares fit to a non-linear
;	function, pages 237-239, Bevington, Data Reduction and Error
;	Analysis for the Physical Sciences.
;
;	"This method is the Gradient-expansion algorithm which
;	compines the best features of the gradient search with
;	the method of linearizing the fitting function."
;
;	Iterations are perform until the chi square changes by
;	only 0.1% or until 20 iterations have been performed.
;
;	The initial guess of the parameter values should be
;	as close to the actual values as possible or the solution
;	may not converge.
;
; MODIFICATION HISTORY:
;	Written, DMS, RSI, September, 1982.
;
;	Renamed ARRAY to ARRAY1 to avoid conflict with ARRAY function;
;		Ian Grant, HIA/NRCC, 9 October 1990.
;
;	Added ERR parameter; Ian Grant, NRC, 18 October 1990.
;
;	Added IPAR parameter. Based on Don Wallis's DDWFIT of 29/10/90. DDWFIT
;	also included double precision and modified criteria for termination of
;	the convergence search which are not included here.
;	Ian Grant, NRC, 29 October 1990.
;
;	IFG  6 Feb 1991: 
;		Converted to IDL2.
;		Added DMS Oct 1990 change: Does not iterate if the first guess 
;		   is good.
;		Didn't implement DMS keyword FUNCTION_NAME, since I couldn't 
;		   find procedure CALL_PROCEDURE.
;
;	IFG  4 Jun 1991: Rounded NTERMS to an integer, since TOTAL always float.
;
;	IFG 16 Sep 1991: Added check for Q non-empty before BB(Q) = A(Q).
;
;	IFG  5 JAN 1993: Renamed from CURVEFIT to CURVEFIT_IFG.
;		Noted some enhancements added to CURVEFIT in IDL user 
;		library since this version was created by IFG in October 1990.
;-

FUNCTION CURVEFIT_IFG,X,Y,W,A,IPAR,SIGMAA,ERR
	ON_ERROR,2		;RETURN TO CALLER IF ERROR
        ERR = 0						; IFG 18-Oct-1990
	A = 1.*A		;MAKE PARAMS FLOATING
	NPAR = N_ELEMENTS(A)				; IFG 29-Oct-1990
	NTERMS = FIX(TOTAL(IPAR)+0.5)	;# OF PARAMS.	; IFG 29-Oct-1990,910604
	NFREE = (N_ELEMENTS(Y)<N_ELEMENTS(X))-NTERMS ;Degs of freedom
	IF NFREE LE 0 THEN STOP,'Curvefit - not enough data points.'
	P = WHERE(IPAR EQ 1)				; IFG 29-Oct-1990
	Q = WHERE(IPAR EQ 0)				; IFG 29-Oct-1990
	FLAMBDA = 0.001		;Initial lambda
	DIAG = INDGEN(NTERMS)*(NTERMS+1) ;SUBSCRIPTS OF DIAGONAL ELEMENTS
;
	FOR ITER = 1,20 DO BEGIN	;Iteration loop
;
;		EVALUATE ALPHA AND BETA MATRICIES.
;
	FUNCT,X,A,YFIT,DER	;COMPUTE FUNCTION AT A.	; IFG 29-Oct-1990
	PDER = FLTARR(N_ELEMENTS(X),NTERMS)		; IFG 29-Oct-1990
	FOR I=0,NTERMS-1 DO PDER(0,I) = DER(*,P(I))	; IFG 29-Oct-1990
	BETA = (Y-YFIT)*W # PDER
	ALPHA = TRANSPOSE(PDER) # (W # (FLTARR(NTERMS)+1)*PDER)
;
	CHISQ1 = TOTAL(W*(Y-YFIT)^2)/NFREE ;PRESENT CHI SQUARED.
;* DMS Oct 1990 change added by IFG 6 Feb 1991				;*
				; If a good fit, no need to iterate	;*
	if chisq1 lt total(abs(y))/1e7/NFREE then begin			;*
		sigmaa = fltarr(nterms)	;Return all 0's			;*
		return, yfit						;*
		endif							;*

;
;	INVERT MODIFIED CURVATURE MATRIX TO FIND NEW PARAMETERS.
;
	REPEAT BEGIN
		C = SQRT(ALPHA(DIAG) # ALPHA(DIAG))
		ARRAY1 = ALPHA/C
		ARRAY1(DIAG) = 1.+FLAMBDA
		ARRAY1 = INVERT(ARRAY1)
		B = A(P) + ARRAY1/C # TRANSPOSE(BETA) ;NEW PARAMS ; IFG 29-Oct-1990
		BB = FLTARR(NPAR)			; IFG 29-Oct-1990
		BB(P) = B				; IFG 29-Oct-1990
		IF ISARRAY(Q) THEN BB(Q) = A(Q)	; IFG 29-Oct-1990, 16-Sep-1990
		FUNCT,X,BB,YFIT		;EVALUATE FUNCTION ; IFG 29-Oct-1990
		CHISQR = TOTAL(W*(Y-YFIT)^2)/NFREE ;NEW CHISQR
		FLAMBDA = FLAMBDA*10.	;ASSUME FIT GOT WORSE
		ENDREP UNTIL CHISQR LE CHISQ1
;
	FLAMBDA = FLAMBDA/100.	;DECREASE FLAMBDA BY FACTOR OF 10
	A=BB			;SAVE NEW PARAMETER ESTIMATE. ; IFG 29-Oct-1990
;	PRINT,'ITERATION =',ITER,' ,CHISQR =',CHISQR
;	PRINT,A
	IF ((CHISQ1-CHISQR)/CHISQ1) LE .001 THEN GOTO,DONE ;Finished?
	ENDFOR			;ITERATION LOOP
;
	IF N_PARAMS(0) LT 6 THEN message, 'Failed to converge', /INFORMATIONAL
        ERR = 1						; IFG 18-Oct-1990
;
DONE:	SIGMAA = SQRT(ARRAY1(DIAG)/ALPHA(DIAG)) ;RETURN SIGMA'S
	RETURN,YFIT		;RETURN RESULT
END
