;+
; NAME:
;    CAN_HODOGRAMS
; PURPOSE:
;    Plot hodogram (polarization traces) of (x,y) as a function of CANOPUS time.
;    The time interval may be broken into shorter segmaents to reduce confusion
;    of the trace.
; CALLING SEQUENCE:
;    can_hodograms, times, x, y
; INPUTS:
;    TIMES = LONG vector of CANOPUS times
;    X     = FLOAT vector of X values. Should be same length as TIMES.
;    Y     = FLOAT vector of Y values. Should be same length as TIMES.
; KEYWORD PARAMETERS:
;    TPANEL = LONG scalar. Interval in seconds covered by each panel (subplot).
;             Default: plot all data on a single panel.
;    TTICK  = Long scalar. Time in seconds between tick marks along the trace. 
;             Each tick is actually placed at the [X(i), Y(i)] point 
;             corresponding to the first T(i) which is greater than or equal
;             to a multiple of TTICK. Default is no ticks.
;    TITLE  = STRING scalar. Main plot title. Default: ' '.
;    XTITLE = STRING scalar. X title for each panel. Default: 'X'.
;    YTITLE = STRING scalar. Y title for each panel. Default: 'Y'.
;    MAXAMP = Maximum deflection from zero in X or Y to fit on panel.
;             Default: ROUNDUP_NICE(MAX(ABS([X,Y])))
;    MAX_VALUE = Numeric scalar. Any (X,Y) pair in which either X or Y is 
;             greater than MAX_VALUE is treated as a missing data point.
; SIDE EFFECTS:
;    Produces graphics output.
; PROCEDURE:
;	Calls REGION_MULTI to create subplots.
; MODIFICATION HISTORY:
;    Ian Grant, HIA/NRCC, 13 Jan 1992: Written in IDL2.
;	IFG, 15 Dec 1992: Added ON_ERROR.
;	IFG, 22 Dec 1992: Replaced ASPECTD_XONY call by ASPECT.
;       GRANT, 28 Sep 1993: Changed REGION_MULTI calls to suit new definition
;          of POSITION argument.
;	GRANT, 23 Jun 1994: Added RETURN statement!
;          Added MAX_VALUE and TTICK keywords.
;	   Added plotting of a dashed line across gaps.
;-

pro can_hodograms, times, x, y, tpanel=tpanel, ttick=ttick, title=title_in, $
   xtitle=xtitle, ytitle=ytitle, maxamp=maxamp, max_value=max_value

;%% Plot start and end markers on last NONMISSING point, so they are never 
;%% omitted.

on_error, 2
psav = !p

pos_regions = [0.05, 0.05, 1.0, 0.92]
pos_in_region = [0.1, 0.1, 0.9, 0.9]

if not keyword_set(max_value) then max_value = 99990.0

ntimes = n_elements(times)
w_nonmissing = where( (x le max_value) and (y le max_value), n_nonmissing)

if not keyword_set(tpanel) then tpanel = times(ntimes-1) - times(0)
if not keyword_set(title_in)  then title_in = ' '
if not keyword_set(xtitle) then xtitle = 'X'
if not keyword_set(ytitle) then ytitle = 'Y'
if not keyword_set(maxamp) then $
   if n_nonmissing gt 0 then $
      maxamp = roundup_nice(max(abs([x(w_nonmissing), y(w_nonmissing)]))) $
   else $
      maxamp = 1.0

title = title_in
if keyword_set(ttick) then $
   title = title + '    Trace tick spacing = ' + strim(ttick) + ' s'

; Determine number and arrangement of panels

npanels = roundup( float(times(ntimes-1) - times(0)) / float(tpanel) )
aspectmain_xony = aspect(/device) / $
   ((pos_regions(3) - pos_regions(1)) / (pos_regions(2) - pos_regions(0)))
aspectsub_xony = 1.0
layout_subplots, aspectmain_xony, aspectsub_xony, npanels, nx, ny
case !true of
   ny le 2: plot_sizefact = 1.0
   ny eq 3: plot_sizefact = 0.75
   else:    plot_sizefact = 0.5
   endcase

!p.noerase = 1
erase
!p.charsize = 1.0
region_multi, /init, [nx,ny], pos=pos_regions, title=title
!p.charsize = plot_sizefact
for panelx=0,npanels-1 do begin

   panel_timerange = times(0) + tpanel * [panelx, panelx+1]
   time_subs = $
      where( (times ge panel_timerange(0)) and (times le panel_timerange(1)) )
   ; Is there a tidier or more efficient way to find TIME_SUBS_NONMISS?
   ; Want intersection of TIME_SUBS and W_NONMISSING.
   time_subs_nomiss = $
      where( (times ge panel_timerange(0)) and (times le panel_timerange(1)) $
         and (x le max_value) and (y le max_value) )

   region_multi, panelx
   region_position, pos_in_region
   setmap, xrange=[-maxamp,maxamp], yrange=[-maxamp,maxamp] ;%% 940825

   if (panelx mod nx) eq 0 then $
      ytitle_out = ytitle $
   else $
      ytitle_out = ' '
   if (npanels-panelx) le nx then $
      xtitle_out = xtitle $
   else $
      xtitle_out = ' '

   plot, [0,0], /nodata, title=strcanrange(times(time_subs), /time), $	; Axes
      xtitle=xtitle_out, xstyle=1, xrange=[-maxamp,maxamp], $
      ytitle=ytitle_out, ystyle=1, yrange=[-maxamp,maxamp]
   plots, [0], [0], psym=1				; Mark origin with plus
   oplot, x(time_subs), y(time_subs), max_value=max_value	; Plot hodogram
   if n_nonmissing lt ntimes then $	; Oplot dashed line across gaps
      oplot, x(time_subs_nomiss), y(time_subs_nomiss), linestyle=1
   plots, x(time_subs(0)), y(time_subs(0)), psym=7	; Mark start with square
   plots, x(last(time_subs)), y(last(time_subs)), psym=6   ; Mark end with X
   if keyword_set(ttick) and (n_elements(time_subs) ge 3) then begin
      ; Mark ticks on trace at intervals of TTICK, using small pluses
      times_down2tick = ttick * long(times(time_subs) / ttick)
      ticktime_subs = time_subs( where( difference(times_down2tick) gt 0 ) +1 )
      plots, x(ticktime_subs), y(ticktime_subs), psym=1, symsize=0.5
      endif
   endfor ; panelx

region_multi, /reset
!p = psav
return

end
