;+
; NAME:
;	CAN_SONOGRAMS
; PURPOSE:
;	Plot stacked dynamic spectra against CANOPUS time.
;	For each time series (data channel) the MEM spectrum is calculated in a 
;	data window which is stepped along the time series. The result is
;	displayed as an image on frequency versus time axes, each vertical
;	strip of the image being the spectrum for the data window centred on 
;	that time.
; CALLING SEQUENCE:
;	CAN_SONOGRAMS, TIMES, Y
; INPUTS:
;	TIMES = Vector of CANOPUS times. Length is ntimes.
;		May be integer or floating, but if floating then some checks are
;		skipped.
;	Y = 2D array, dimensions (ntimes, nchans). Each Y(*,i) is a time series.
; KEYWORD PARAMETERS:
;	WLEN = Length of data window. Units are those of TIMES.
;		If absent or zero, the user is prompted for it and the entered 
;		value is passed back to the calling program. Integer or float 
;		to match TIMES.
;	WRANGE = 2 element vector of CANOPUS times. Centre times of first
;		and last positions of the data window. Units are those of TIMES.
;		If absent or zero, the user is prompted for it and the entered 
;		value is passed back to the calling program. Integer or float 
;		to match TIMES, but for float you must set it by keyword 
;		rather than in response to the prompt.
;	WSTEP = Shift of data window between spectra. Units are those of
;		TIMES.
;		If absent or zero, the user is prompted for it and the entered 
;		value is passed back to the calling program. Integer or float 
;		to match TIMES.
;	MAX_VALUE: Data >= MAX_VALUE are replaced by interpolation. Default is
;		99990.0 (for MARIA).
;	YTITLES = Vector of strings. Y-axis title for each channel.
;	TITLE = String title for page.
;	INFORM: If present and non-zero, progress messages are displayed.
;	POSITION = 4-element vector. Position in normalized coordinates of
;		rectangle enclosing all plots, excluding labels.
;	MAXFREQ = Scalar. Maximum frequency to plot in mHz.
;	NFREQS = Scalar integer. Number of frequencies to plot. Default 101.
;	ORDER = Scalar integer. Order of MEM spectrum. Zero selects default.
;	LEVELS_ARR = 1D or 2D array of levels for color/greyscale.
;		1D: Same levels for each channel. If scalar NLEVELS_ARR is 
;		   present and non-zero, first NLEVELS_ARR elements are used.
;		2D: Each row is levels for one channel. If NLEVELS_ARR is 
;		   vector of length equal to number of channels, then
;		   LEVELS_ARR(0:NLEVELS_ARR(chanx)-1, chanx) used for channel
;		   chanx. If NLEVELS_ARR scalar, then
;		   LEVELS_ARR(0:NLEVELS_ARR-1,chanx) used for channel
;		   chanx.
;		See CANTIME_IMAGES documentation for relation of colours to 
;		   levels.
;	NLEVELS_ARR = Scalar or vector number of levels for each channel.
;	IMAGETYPE = 'c' for filled contours, 'p' for rectangular pixels.
;	CONTOURS: If present and non-zero, filled contours will be used.
;	PIXELS:   If present and non-zero, rectangular pixels will be used.
;	Only one or none of IMAGETYPE, CONTOURS or PIXELS may be set. Default 
;	   is filled contours.
;	FFT:      If present and non-zero, the returned spectrum is the 
;	   amplitude of the FFT. Otherwise an MEM spectrum is returned.
;	DETREND:  The state of this keyword (present and non-zero, or otherwise)
;	   is passed to the SPECTRUM routine.
;	HANNING:  The state of this keyword (present and non-zero, or otherwise)
;	   is passed to the SPECTRUM routine.
;	DB:       If present and non-zero, the spectra are converted to dB.
; SIDE EFFECTS:
;	Graphics output.
; PROCEDURE:
;	Calls MOVING_WINDOW, then SPECTRUM to calculate MEM or FFT spectra,
;	then CANTIME_IMAGES to display.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, July 1992: Written in IDL2.
;	IFG 15 Dec 1992: Added ON_ERROR.
;	IFG, 4 Feb 1993: Renamed MISSING to MAX_VALUE.
;	GRANT, 29 Jun 1993: Set Z titles to 'dB'.
;	GRANT, 15 Sep 1993: Added keywords CONTOURS, PIXELS.
;	GRANT,  6 Jan 1994: Added keyword NFREQS.
;	GRANT,  7 Jan 1994: Changed to use /INDEX in MOVING_WINDOW call.
;	   Allowed TIMES to be floating as well as integer.
;	GRANT, 12 May 1994: Added keywords DETREND, HANNING, FFT, DB.
;-

pro can_sonograms, times, y, $
   wlen=window_len, wrange=window_range, wstep=window_step, $
   max_value=max_value, ytitles=ytitles, title=title_in, inform=inform, $
   position=position, maxfreq=maxfreq, nfreqs=nfreqs, order=order, $
   levels_arr=levels_arr, nlevels_arr=nlevels_arr, $
   imagetype=imagetype_in, contours=contours, pixels=pixels, $
   detrend=detrend_on, hanning=hanning_on, fft=fft, db=db_on

on_error, 2
psav = !p
xsav = !x
ysav = !y

if not keyword_set(nfreqs) then nfreqs = 101

sy = size(y)
case sy(0) of 
   1: nchans = 1
   2: nchans = (size(y)) (2)
   endcase

integer_time = in(datatype(times), ['BYT', 'INT', 'LON'])
if integer_time then $
   window_parm_typ = 'INTEGER' $
else $
   window_parm_typ = 'FLOAT'
ntimes = n_elements(times)
delta_time = (times(ntimes-1)-times(0)) / (ntimes-1)

if not keyword_set(max_value) then max_value = 99990.0	; MARIA missing value
if not keyword_set(ytitles)   then ytitles = strarr(nchans)
if not keyword_set(inform)    then inform = 0
if not keyword_set(position)  then position = 0

mem = !true	;%% 940512
spectrum_type = 'MEM'
if keyword_set(fft) then begin
   mem = !false
   spectrum_type = 'FFT'
   endif

if keyword_set(db_on) then $
   ztitle = 'dB' $
else $
   ztitle = 'Ampl.'

case howmany( [ keyword_set(imagetype_in), keyword_set(contours), $
                keyword_set(pixels) ]) of 
   0: imagetype_in = 'c'
   1: if not keyword_set(imagetype_in) then begin
         if keyword_set(contours) then $
            imagetype_in = 'c' $
         else $
            imagetype_in = 'p'
         endif
   else: message, 'Must set no more than one of IMAGETYPE_IN, CONTOURS, PIXELS'
   endcase
imagetype = strupcase(imagetype_in)

; Select window length

if not keyword_set(window_len) then $
   window_len = enterdef( window_parm_typ, $
      'Window length in same units as sampling interval', 30*60)

; Select window centre times

if not keyword_set(window_range) then begin
   print, ' '
   print, 'Enter window_times start and end'
   window_range = enter_canrange(ends(times))
   endif

; Select window step

if not keyword_set(window_step) then begin
   print, ' '
   window_step = enterdef(window_parm_typ, 'Window step', window_len)
   endif
if integer_time then $
   if (window_step mod delta_time) ne 0 then $
      message, 'Window step is not a multiple of the sampling interval'

; Choose every (WINDOW_STEP/DELTA_TIME)th TIMES within range given by
; WINDOW_RANGE, starting with the first in range.
w = where((times ge window_range(0)) and (times le window_range(1)), $
   wcount)
if wcount eq 0 then $
   message, 'No data in selected time range'
window_timexs = $
   w( where((indgen(wcount) mod round(window_step/delta_time)) eq 0) )
window_times = times(window_timexs)

; Enter order of MEM spectrum

if (spectrum_type eq 'MEM') and (datatype(order) eq 'UND') then $
   order = enter('INTEGER', 'Order of MEM spectrum (<return> for default)', 0)

; Enter maximum frequency MAXFREQ in mHz

nyqfreq = 1000.0*0.5/delta_time		; In mHz
if not keyword_set(maxfreq) then begin
   repeat $
      maxfreq = enterdef('FLOAT', 'Maximum frequency (mHz)', nyqfreq) $
      until maxfreq le nyqfreq
   endif
if maxfreq gt nyqfreq then $
   message, 'MAXFREQ must be LE Nyquist frequency'

nfsteps_param = nfreqs-1	;%% 940512
if spectrum_type eq 'FFT' then nfsteps_param = 0	;%% 940512

; Setup greyscale levels, or force default to be selected later

if datatype(levels_arr) eq 'UND' then begin
   levels_arr = 0  ; Force CANTIME_IMAGES to prompt for this before screen plot
   nlevels_arr = 0 ; "
   endif $
else begin ; LEVELS_ARR defined
   s_levels_arr = size(levels_arr)
   case s_levels_arr(0) of
      1: begin	; 1D LEVELS_ARR
         if keyword_set(nlevels_arr) then $
            nlevels_arr = replicate(nlevels_arr(0), nchans) $
         else $
            nlevels_arr = replicate(s_levels_arr(1), nchans)
         levels_arr_in = levels_arr
         levels_arr = fltarr(nlevels_arr(0), nchans)
         for chanx=0,nchans-1 do $
            levels_arr(0,chanx) = levels_arr_in(0:nlevels_arr(chanx)-1)
         end
      2: begin	; 2D LEVELS_ARR
         if s_levels_arr(2) ne nchans then $
            message,'LEVELS_ARR second dimension must equal number of channels'
         if keyword_set(nlevels_arr) then begin
            s_nlevels_arr = size(nlevels_arr)
            if (s_nlevels_arr(0) ne 1) or (s_nlevels_arr(1) ne nchans) then $
               message, $
  'NLEVELS_ARR must be a scalar or vector of length equal to number of channels'
            endif $
         else $
            nlevels_arr = replicate(s_levels_arr(1), nchans)
         end
      else: message, 'LEVELS_ARR must have 1 or 2 dimensions'
      endcase
   endelse ; LEVELS_ARR defined

; Calculate and save all sonograms.
; Could calculate all spectra without CHANX loop, but must have the loop for 
; TRANSPOSE and it allows INFORM messages.

for chanx=0,nchans-1 do begin
   if inform then begin
      print, ' '
      print, 'Calculating dynamic spectrum for ' + ytitles(chanx)
      endif

   ; Fix gaps

   y1 = y(*,chanx)
   fill = max_value
   ibad=where(y1 ge max_value, nbad)
   if nbad gt 0 then begin
      message, /informational, 'Interpolating across missing points'
      y1(ibad) = fill
      fixgaps, y1, fill, 1
      endif

   ; Apply moving window

   yw = moving_window(y1, times, window_timexs, round(window_len/delta_time), $
      /index)

   ; Calculate spectra for all windows. Delta_time in kiloseconds to get mHz.
   ; REFORM eliminates trailing dimension of 1.

   s_onechan = reform( spectrum(yw, dtime=delta_time/1000.0, $
      mem=mem, fft=keyword_set(fft), order=order, $	;%% 940526
      detrend=keyword_set(detrend_on), hanning=keyword_set(hanning_on), $ ;%% 940512
      fmax=maxfreq, nfsteps=nfsteps_param, f_out=freqs, inform=inform) )
   if keyword_set(db_on) then $
      s_onechan = db(s_onechan)

   if chanx eq 0 then $
      s = fltarr(n_elements(window_timexs), n_elements(freqs), nchans)
   s(0,0,chanx) = transpose(s_onechan)	; Put window_time first for x-axis
   endfor ; chanx

; Build title.
; After calculating sonograms to get ORDER value in case default chosen.

case spectrum_type of
   'MEM': spectrum_detail = ', order=' + strim(order)
   'FFT': spectrum_detail = ''
   endcase
if keyword_set(detrend_on) then $
   spectrum_detail = spectrum_detail + ', detrended'
if keyword_set(hanning_on) then $
   spectrum_detail = spectrum_detail + ', Hanning'

title = strcanrange(window_times) + $
   '!c' + spectrum_type + $
   ' dynamic spectrum (!4D!3t=' + strim_float(delta_time) + spectrum_detail + $
   ', window: len=' + strim(window_len) + ', step=' + strim(window_step) + ')'
if keyword_set(title_in) then $
   title = title_in + '    ' + title

; Plot all sonograms

freqrange = ends(freqs, extend_cell=extend_cell)
if nchans le 4 then $
   funit_text = '!cFrequency (mHz)' $
else $
   funit_text = '!cmHz'
!p.noerase = 1
repeat begin ; 'Adjust scales and replot?'
   erase
   repeat begin ; hardcopy()
      time
      cantime_images, s, window_times, freqs, position=position, $
         title=title, ytitles=ytitles+funit_text, $
         ztitles=replicate(ztitle,nchans), $
         levels_arr=levels_arr, nlevels_arr=nlevels_arr, imagetype=imagetype
      endrep until hardcopy()
   levels_arr = 0  ; Force CANTIME_IMAGES to prompt for this before screen plot
   nlevels_arr = 0 ; "
   endrep until reply_char('Adjust scales and replot? (y/n)', 'yn', 'n') eq 'N'

!p = psav
!x = xsav
!y = ysav

end
