;+
; NAME:
;	ENTER_CANRANGE
; PURPOSE:
;	Enter start and end CANOPUS times as YrMoDaHrMnSc.
;	Entered times must be within or equal to range of LIMITS_IN.
;	Entered start time must be LT entered end time.
; CALLING SEQUENCE:
;	Result = ENTER_CANRANGE( [LIMITS_IN] )
; OPTIONAL INPUT PARAMETERS:
;	LIMITS_IN = Long integer vector of length 2. Start & end CANOPUS times.
;	   If LIMITS_IN is present then start time must be GE first element of 
;	   LIMITS_IN, and end time must be LE last element of LIMITS_IN.
; OUTPUTS:
;	Result = Long integer vector of length 2. Start & end CANOPUS times.
; PROCEDURE:
;	Calls ENTER_CANTIME with appropriate prompts.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 6 Sep 1991, IDL2.
;       IFG, 24 Jul 1992: Added ON_ERROR.
;-

function enter_canrange, limits_in

on_error, 2
if keyword_set(limits_in) then begin
   limits = ends(limits_in)	; Only want first and last elements
   print, ' '
   print, 'Start time:  ' + strcantime(limits(0))
   print, 'End time:    ' + strcantime(limits(1))
   print, ' '
   endif $
else $
   limits = [-999999999L, 999999999L]	; Absurdly small and large CANOPUS times
repeat begin
   if n_elements(stime) gt 0 then begin
      ; Signal error since not first attempt
      bell
      message, /inform, 'Start time out of range'
      print, ' '
      endif
   stime = enter_cantime('Start time')
   endrep until (stime ge limits(0)) and (stime lt limits(1))
print, ' '
repeat begin
   if n_elements(etime) gt 0 then begin
      ; Signal error since not first attempt
      bell
      message, /inform, 'End time out of range'
      print, ' '
      endif
   etime = enter_cantime('End time  ')
   endrep until (etime gt stime) and (etime le limits(1))
return, [stime, etime]
end
;+
; NAME:
;	ENTER_CANTIME
; PURPOSE:
;	Enter CANOPUS time as YrMoDaHrMnSc or Yr-DayHrMnSc.
;	Blanks in the 12 character input field are replaced by zeros.
;	Re-entry requested if time is invalid.
; CALLING SEQUENCE:
;	Result = ENTER_CANTIME(PROMPT)
; OPTIONAL INPUT PARAMETERS:
;	PROMPT = String used as prompt. Default is null string.
; KEYWORD PARAMETERS:
;	DEFAULT_CANTIME = LONG scalar. Input default CANOPUS time.
; OUTPUTS:
;	Result = LONG scalar. CANOPUS time.
; PROCEDURE:
;	Calls ENTER, edits response string, and calls CANTIME.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 6 Sep 1991, IDL2.
;       IFG, 24 Jul 1992: Added ON_ERROR.
;-

function enter_cantime, prompt, default_cantime=default_cantime

on_error, 2
if n_params() lt 1 then prompt = ''
if keyword_set(default_cantime) then begin
   civil_time, year, month, date, day, hour, minute, second, default_cantime
   default_reply = string((year mod 100), month, date, hour, minute, second, $
      format='(6i2.2)')
   endif $
else $
   default_reply = ''

repeat begin ; not ct_error
   prompt_line = '  YrMoDaHrMnSc  ' + prompt
   if keyword_set(default_cantime) then $
      prompt_line = $
         prompt_line + '    [Default: ' + strcantime(default_cantime) + ']'
   print, prompt_line
   str = enter('STRING', '', default_reply, dummy, $
      ['YrMoDaHrMnSc for month and 2-digit date of month', $
       'Yr-DayHrMnSc for 3-digit day of year', $
       'Trailing zeroes may be omitted'] )
   str = stress(str, 'R', 0, ' ', '0')	   ; Replace all spaces with zeroes
   str = strmid(str+'000000000000', 0, 12) ; Right fill with zeroes to length=12

   yr = 0L
   mo = 0L
   da = 0L
   hr = 0L
   mn = 0L
   sc = 0L
   if strmid(str,2,1) eq '-' then $
      reads, str, yr, da, hr, mn, sc, format='(i2,x,i3,3i2)' $
   else $
      reads, str, yr, mo, da, hr, mn, sc, format='(6i2)'
   time = cantime(1900+yr, mo, da, hr, mn, sc, ct_error)
   if ct_error then begin
      bell
      print, ' '
      endif
   endrep until not ct_error
return, time
end
