;+
; NAME:
;	NICE_TICKS
; PURPOSE:
;	Return round values (for ticks) which span the range of input values.
; CALLING SEQUENCE:
;	Result = NICE_TICKS( A, NINTERVALS )
; INPUTS:
;	A = Numeric scalar or vector. Only MIN(A) and MAX(A) are used.
;	NINTERVALS = Integer. Approximate maximum number of tick intervals
;	(corresponding to about NINTERVALS+1 tick values). Default is 8.
; OUTPUTS:
;	Result = A float vector of equally spaced values such that the first
;	is (a little) less than MIN(A) and the last is (a little) greater 
;	than MAX(A), and the interval between ticks is of the form m*10^n,
;	where m=1,2, or 5.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, Oct-1990: Written in IDL1.
;	IFG,  7 Feb 1991: No changes needed for IDL2.
;	GRANT, 20 Sep 1993: Replaced ROUNDDOWN/UP by FLOOR/CEIL. Put ROUND in 
;	   calculation of number of ticks, in RETURN statement.
;-

function nice_ticks, a,nintervals

on_error, 2
if n_params(0) le 1 then nintervals = 8		; For contour plots
nticks = nintervals + 1

mina = float(min(a))
maxa = float(max(a))
rangea = maxa-mina
if rangea eq 0.0 then begin
   ; Data is constant
   if mina ne 0.0 then begin
      ; Constant and non-zero: Use +/- 10%
      mina = mina - 0.1*abs(mina)
      maxa = mina + 0.1*abs(mina)
   endif else begin
      ; Constant and zero: Use +/- 1
      mina = -1.0
      maxa = 1.0
      endelse
   rangea = maxa-mina	; Recalculate
   endif
power = 10.0 ^ floor(alog10(rangea))
mantissa = rangea/power
; "-2" determined by experiment.
; For nintervals=7 (nticks=8), using -2 produces 6 to 8 ticks.
mantissa = mantissa / (nticks-2)

if mantissa le 0.2 then mantissa = 0.2 $
else if mantissa le 0.5 then mantissa = 0.5 $
else if mantissa le 1.0 then mantissa = 1.0 $
else if mantissa le 2.0 then mantissa = 2.0 $
else if mantissa le 5.0 then mantissa = 5.0 $
else mantissa = 10.0

delta_level = mantissa * power
min_level = delta_level * floor(mina / delta_level)
max_level = delta_level * ceil (maxa / delta_level)

return, min_level + delta_level* $
   findgen(round( (max_level-min_level)/delta_level ) + 1)

end
