;+
; NAME:
;	POSNEG_TICKS
; PURPOSE:
;	Return uniformly spaced round values centred on zero, spanning argument.
;	Result vector could be used as tick values for an axis or steps on a 
;	color scale, centred on zero.
; CALLING SEQUENCE:
;	Result = POSNEG_TICKS( X, HALFN_INTERVALS )
; INPUTS:
;	X = Numeric input array which the values of Result must span.
;	   Only MAX(ABS(X)) is used.
; OPTIONAL INPUT PARAMETERS:
;	HALFN_INTERVALS = Number of ticks (or intervals) on one side of zero.
;	   Default is 4.
; KEYWORD PARAMETERS:
;	NOZERO: If present and nonzero, the zero value is omitted from the
;	   middle of Result.
; OUTPUTS:
;	Result = Vector of tick values.
;	   If NOZERO is not set, length is 2*HALFN_INTERVALS+1 and the middle
;	   value Result(HALFN_INTERVALS+1) is zero. 
;	   If NOZERO is set, length is 2*HALFN_INTERVALS and the zero is 
;	   omitted.
;	   Other values are of the form +/- m*10^n, where m = 1, 2 or 5.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 23 Sep 1993: Written.
;-

function posneg_ticks, a, halfn_intervals, nozero=nozero

on_error, 2
if not keyword_set(halfn_intervals) then halfn_intervals = 4

maxa = float(max(abs(a)))
delta = roundup_125( maxa / halfn_intervals )	; Spacing of result values
halfn_int_out = ceil (maxa/delta)	; # steps needed to span 0 to MAXA
if keyword_set(nozero) then begin
   pos_indices = 1+lindgen(halfn_int_out)	; 1, ..., n
   return, delta * [ -reverse(pos_indices), pos_indices ]
   endif $
else $
   return, delta * (-halfn_int_out + findgen(2*halfn_int_out + 1) )

end
