;+
; NAME:
;	STRIM_FLOAT
; PURPOSE:
;	Convert a number to a string, omitting non-significant zeroes and point.
;	Convert a number to a string without leading or trailing spaces, and
;	remove '0's and '.' from mantissa and exponent if present to give
;	minimum length string consistent with keeping all significant figures
;	and place-holding zeroes.
; CALLING SEQUENCE:
;	Result = STRIM_FLOAT( NUM )
; INPUTS:
;	NUM = Real scalar or vector.
; KEYWORD PARAMETERS:
;	FORMAT = Scalar string. Format specifier used to convert NUM to string.
; OUTPUTS:
;	Result = String with same dimensions as NUM. NUM converted to string.
; RESTRICTIONS:
;	Need to allow for upper case 'E' for exponent? (and 'd' and 'D' also?)
; MODIFICATION HISTORY:
;	Ian Grant, Feb 1991. IDL2.
;	IFG, 21 May 1991: Added handling of array NUM.
;	IFG, 24 Dec 1992: Added ON_ERROR. Fixed parameter handling which had
;	   treated FORAMT keyword as a second non-keyword parameter.
;-

function strim_float, num, format=format_string

if keyword_set(format_string) then $
   strs = strim(float(num),format=format_string) $
else $
   strs = strim(float(num))

strs_out = strs		; For size
for numx=0,n_elements(num)-1 do begin	; IFG Added 21 May 1991
str = strs(numx)

; Separate into mantissa and any exponent which exists

exp_pos = strpos(str,'e')
if exp_pos eq -1 then begin
   mant = str
   exp = ''
endif else begin
   mant = strmid(str,0,exp_pos)
   exp = strmidend(str,exp_pos)
   endelse

mant = strtrm2(mant,2,'0')	; Trim leading and trailing zeroes
if strmid(mant,0,1) eq '.' then mant = '0'+mant	; Ensure a digit before point
mant = strtrm2(mant,0,'.')	; Trim trailing decimal point

; Trim leading zeroes from integer in exponent
if exp ne '' then exp = strmid(exp,0,2) + strtrm2(strmidend(exp,2),1,'0')
; Throw away zero exponent
if strlen(exp) eq 2 then exp = ''

strs_out(numx) = mant + exp
endfor ; numx

if isarray(num) then $
   return, strs_out $
else $
   return, strs_out(0)

end
