;+
; NAME:
;	UNWRAP
; PURPOSE:
;	Remove data jumps due to the ends of a circular scale (e.g. 360 degs).
;	The minimum of the result has the smallest non-negative value of those
;	allowed.
; CALLING SEQUENCE:
;	Result = UNWRAP( DATA_IN, MAXVAL)
; INPUTS:
;	DATA_IN = Vector of data which might contain jumps.
;	MAXVAL  = Max of range of DATA_IN values. Range is 0.0 to MAXVAL.
; OUTPUTS:
;	Result = Data vector with range possibly exceeding 0.0 to MAXVAL.
;	   Elements of Result differ from corresponding elements of DATA_IN by 
;	   integral multiples of MAXVAL. MIN(Result) is in the range 0.0 to 
;	   MAXVAL.
; RESTRICTIONS:
;	Any increment in DATA_IN exceeding +/- 0.5*MAXVAL is treated as a jump 
;	caused by the value sequence crossing an end of the range 0.0 to MAXVAL.
; PROCEDURE:
;	The input vector is scanned from start to end.
;	At any increment outside +/- 0.5*MAXVAL, to bring the increment within
;	range the procedure increments or decrements the number of multiples of
;	MAXVAL needed to be applied to the remainder of the vector.
;	Then the whole vector is shifted to bring the minimum into the range 0 
;	to MAXVAL.
; MODIFICATION HISTORY:
;	Written: Ian Grant, HIA/NRCC, 1990.
;	Converted to IDL2: Ian Grant, 1991.
;	IFG, 5 Jan 1993: Renamed from DEDRUM to UNWRAP. Added ON_ERROR.
;-

function unwrap, data_in,maxval

on_error, 2
data = unwrap_noshift(data_in,maxval)

; Shift DC level of data by a multiple of maxval so that minimum of series
; is in range 0..maxval.

c = !c ; Save before MIN
mindata = min(data)
!c = c ; Restore
data = round(posmod(mindata,maxval)-mindata) + data

return, data
end
