;+
; NAME:
;	COLOR_LEGEND
; PURPOSE:
;	Plot a vertical labelled color bar to be used as a legend.
; CALLING SEQUENCE:
;	COLOR_LEGEND, POSITION=POSITION, COLORS=COLORS, LABELS=LABELS_IN, $
;	   TITLE=TITLE [,SIZE=size] [,OPEN=open] [,CLOSED=closed]
; KEYWORD PARAMETERS:
;	POSITION = 4-element vector. Corners of color bar, excluding labels.
;	COLORS = Indices into color table for colors to be plotted.
;	LABELS = String array. Labels for the color boundaries.
;	TITLE = Scalar string. Title to be written above the color bar.
;	SIZE = Scalar. Size of characters for title and labels.
;	OPEN: If present and nonzero, the bottom and top of the bar are not
;	   labelled. In this case N_ELEMENTS(LABELS) must be
;	   N_ELEMENTS(COLORS)-1.
;	CLOSED: If present and nonzero, the bottom and top of the bar are 
;	   labelled. In this case N_ELEMENTS(LABELS) must be
;	   N_ELEMENTS(COLORS)+1.
;	At most one of OPEN and CLOSED can be set. Default is /CLOSED.
;	BORDERS: If present and nonzero, a line is drawn around each 
;	   rectangle in the color bar. Necessary if one of the colors is the 
;	   background color.
; SIDE EFFECTS:
;	Generates graphics output.
; PROCEDURE:
;	Calls TVXY for the colors, XYOUTS for the title and labels.
;	Decimates labels if required to prevent overlap.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 17 May 1991, IDL2.
;	IFG, 22 Dec 1992: Added ON_ERROR.
;	GRANT, 26 May 1994: Added keyword BORDERS.
;-

pro color_legend, position=position, colors=colors, labels=labels_in, $
   title=title, size=size, open=open, closed=closed, borders=borders

on_error, 2
if not keyword_set(position)  then message, 'Must set keyword POSITION'
if not keyword_set(colors)    then message, 'Must set keyword COLORS'
if not keyword_set(labels_in) then message, 'Must set keyword LABELS'
if not keyword_set(title)     then message, 'Must set keyword TITLE'
if not keyword_set(size)      then size = 1.0
labels = labels_in

kc = keyword_set(closed)
ko = keyword_set(open)

if ((not kc) and (not ko)) then closed = !true	; Default
if ((not kc) and      ko ) then closed = !false
if (     kc  and (not ko)) then closed = !true
if (     kc  and      ko ) then message, 'Conflicting keywords OPEN and CLOSED'

ncolors = n_elements(colors)
nlabels = n_elements(labels)

if closed then begin
   if nlabels ne (ncolors+1) then $
      message, 'LABELS dimension must be exactly COLORS dimension + 1'
   endif $
else begin
   if nlabels ne (ncolors-1) then $
      message, 'LABELS dimension must be exactly COLORS dimension - 1'
   endelse

; Find edges (i.e. color boundaries). These are potential label positions.

yedges = findgen(ncolors+1) - 0.5

; Plot color rectangles

xrange=[0.0,4.0]
plot, findgen(2), findgen(2), /nodata, position=position, title='', $
   xrange=xrange, yrange=[yedges(0),yedges(ncolors)], xstyle=5, ystyle=5 
tvxy, transpose([[colors],[colors]]), [1.0,3.0], findgen(ncolors)
if keyword_set(borders) then begin
   for edgex=0,ncolors do $	; Horizontal borders
      plots, xrange, replicate(yedges(edgex), 2)
   for edgex=0,1 do $		; Vertical borders
      plots, replicate(xrange(edgex), 2), yedges([0,ncolors])
   endif
y_char_size_dat = y_ch_size(/data)

; Plot title

xyouts, 0.0, yedges(ncolors)+y_char_size_dat, title, alignment=0.0, /data, $
   size=size

; Find label placements

if closed then $
   labelled_edges = indgen(nlabels) $	; Label all edges
else $
   labelled_edges = 1 + indgen(nlabels)	; Label all edges but first and last
ylabels = yedges(labelled_edges)

; Possibly decimate labels so they are not overlapping one another

if yedges(1)-yedges(0) lt y_char_size_dat then begin
   decim_factor = roundup( y_char_size_dat / (yedges(1)-yedges(0)) )
   decim_indices = where((indgen(nlabels) mod decim_factor) eq 0)
   ylabels = ylabels(decim_indices)
   labels  = labels(decim_indices)
   endif

; Plot labels

for labelx = 0,n_elements(labels)-1 do $
   xyouts, 4.0, ylabels(labelx)-0.5*y_char_size_dat, ' '+labels(labelx), $
      /data, /noclip, size=size

return
end
